package ec.multirun;

import ec.*;
import ec.util.*;
import java.io.*;
import java.util.*;


/**
 * An instance is essentially a table of doubles: it has a table head and keep a bunch of different rows.
 * This class keeps track of its instances, since an instance is going to be used across multiple runs.
 * 
 * @author Gabriel Catalin Balan
 */
public class AnovaDataStorage
{
	public static Vector all = new Vector();
	
	public static AnovaDataStorage getAnovaDataStorage(int i)
	{
		return (AnovaDataStorage)all.get(i);
	}
	
	public static int getId(AnovaDataStorage a)
	{
		return all.indexOf(a);
	}
	
	int currentRun = -1;
	int currentColumn = 0;
	double[] currentdata = null;
	double[/*run*/][/*field*/] data = null;
	String[] header= null;
	
	public AnovaDataStorage(int seeds)
	{
		data = new double[seeds][];
		all.add(this);
	}
	public boolean initialized()
	{
		return header!= null;
	}
	
	public void initialize(Vector v)
	{
		int columns = v.size();
		if(header==null)
		{
			
			header = new String[columns];
			v.copyInto(header);
			for(int i=0;i<data.length; i++)
				data[i] = new double[columns];
		}
		else
			if(header.length != columns) 
				throw new RuntimeException("Invalid number of columns "+columns+", previously initialized at "+header.length);
		currentRun++;
		currentdata = data[currentRun];
		currentColumn = 0;
	}
	
	public void set(int col, double value)
	{
		currentdata[col] = value; //equivalent to data[currentRun][col]=value; 
	}

	public void add(double value)
	{
		currentdata[currentColumn++]=value;
	}
	
	/** for multi-threaded "MultiRun"s */
	public void set(int row, int col, double value)
	{
		data[row][col]=value;
	}
	
	
	

	public void print(final EvolutionState state, int log)
	{
		
		for(int j=0;j<header.length; j++)
			state.output.print(header[j]+"\t",Output.V_NO_GENERAL,log);
		state.output.println("",Output.V_NO_GENERAL,log);			
		for(int i=0;i<data.length; i++)
		{
			for(int j=0;j<data[i].length; j++)
				state.output.print(data[i][j]+"\t",Output.V_NO_GENERAL,log);
			state.output.println("",Output.V_NO_GENERAL,log);
		}
	}
	public void print(final PrintWriter w)
	{
		
		for(int j=0;j<header.length; j++)
			w.print(header[j]+"\t");
		w.println();			
		for(int i=0;i<data.length; i++)
		{
			for(int j=0;j<data[i].length; j++)
				w.print(data[i][j]+"\t");
			w.println();
		}
	}
	
	public double averageOfColumn(int c)
	{
		double sum = 0;
		for(int i=0;i<data.length;i++)
			sum += data[i][c];
		return sum/ data.length;
	}
	
	public double varianceOfColumn(int c){	return varianceOfColumn(c, averageOfColumn(c));}
	public double varianceOfColumn(int c, double average)
	{
		double sum  = 0;
		for(int i=0;i<data.length;i++)
		{
			double err = data[i][c]- average;
			sum += err*err;
		}
		return sum/(data.length-1);
	}
	
	public double confidenceOfColumn(int c, double confidenceLevel)
	{
		if(data.length<30)	
			System.err.println("I've only implemented the '>30' part");
		double alpha = 1-confidenceLevel;
		double sigma = Math.sqrt(varianceOfColumn(c));
		int n = data.length;
		return ProbMath.inverseNormal(1-alpha/2)*sigma/Math.sqrt(n);
	}
	
	public void printConfidenceIntervals(final PrintWriter w, double confidenceLevel)
	{
		for(int j=0;j<header.length; j++)
			w.print(header[j]+"\t");
		w.println();			

		double alpha = 1-confidenceLevel;
		int n = data.length;
		double confidenceOverSigma = ProbMath.inverseNormal(1-alpha/2)/Math.sqrt(n);
		for(int c=0;c<header.length; c++)
			w.print(averageOfColumn(c)+"\t");
		w.println();		
		for(int c=0;c<header.length; c++)
		{
			double sigma = Math.sqrt(varianceOfColumn(c));
			w.print((sigma * confidenceOverSigma)+"\t");
		}
		w.println();
	}
}
