package ec.multirun;
import ec.util.*;

/**
 * Abstract class that chooses a single column to extract from the table;
 * the column is selected based on name. 
 * Implemented behaviors (in the subclasses) include Exact-Match, StartsWith-Match, and EndsWith-Match.
 * Note that only the last column whose name matches the pattern will be used.
 * 
 * @author Catalin Balan
 *
 */
public abstract class HeaderMatchColumnExtractor extends AnovaDataStorageTableShrinker
{
	public final String P_COL = "name-pattern";
	String namePattern;
	
	
	/** set by <code>processHeader(String line)</code> */
	int column;	
	
	public double compute(String line)
	{
		String[] tokens = line.split("\t");
		return Double.parseDouble(tokens[column]);
	}
	
	public void setup(final ParameterDatabase params, final Parameter base)
	{
		namePattern = params.getString(base.push(P_COL),null);
	}
}
