package ec.multirun;

import ec.util.*;
import java.io.*;

/**
 *
 * It takes a bunch of files, it compiles a column out of each file and
 * outputs a tab-spaced table out of them. 
 * Each file's name gets striped of its extension and becomes the column's name.
 * [most common would be .anova.txt, but you can use _sto.anova.txt too]
 * [the extension can contain empty spaces, just not in the beginning, cause they get stripped in ParameterDataBase]
 * 
 * <p>The "extension" parameter is used to filter the files in the current directory.
 * 
 * @author Catalin Balan
 */
public class MultiTreatmentTableMaker
{
	public static final String A_FILE = "-file";
	public static final Parameter P_COLUMN = new Parameter("column");
	public static final String P_COUNT = "count";
	public static final Parameter P_EXTENSION = new Parameter("extension");
	public static final Parameter P_SHRINKER = new Parameter("shrinker");

	String extension = ".anova.txt";
	AnovaDataStorageTableShrinker[] shrinkers;
	
	protected MultiTreatmentTableMaker(ParameterDatabase params) throws IOException
	{
		extension = params.getStringWithDefault(P_EXTENSION, null, extension);
		
		File pwd = new File (".");
		File[] anovaStorageFiles = pwd.listFiles(new FilenameFilter(){public boolean accept(File dir, String name){return name.endsWith(extension);}});
		int n = anovaStorageFiles.length;
		BufferedReader[] readers = new BufferedReader[n];
		shrinkers = new AnovaDataStorageTableShrinker[n];
		for(int i=0;i<n;i++)
		{
			File f = anovaStorageFiles[i];
			String name = f.getName();
			String shortNname = name.substring(0, name.length()-extension.length());
			System.out.print(shortNname+"\t");
			readers[i]=new BufferedReader(new FileReader(f));
			
			shrinkers[i] = (AnovaDataStorageTableShrinker)params.getInstanceForParameter(P_SHRINKER, null, AnovaDataStorageTableShrinker.class);
			shrinkers[i].setup(params, P_SHRINKER);

		}
		System.out.println();
		boolean allDone;
		String line;
		for(int i=0;i<n;i++)
			try
			{
				shrinkers[i].processHeader(readers[i].readLine());
			}catch (Exception e)
			{
				System.err.println("Error in file "+anovaStorageFiles[i].getName());
				e.printStackTrace();
				System.exit(-1);
			}
		do{
			allDone = true;
			for(int i=0;i<n;i++)
				try
				{
					if((line = readers[i].readLine())!=null)
					{
						allDone = false;
						System.out.print(shrinkers[i].compute(line)+"\t");
					}
				}catch (Exception e)
				{
					System.err.println("Error in file "+anovaStorageFiles[i].getName());
					e.printStackTrace();
					System.exit(-1);
				}
			System.out.println();
		}while(! allDone);
		
	}
	public static void main(String[] args) throws IOException
	{
		ParameterDatabase parameters=null;
		for(int x=0;x<args.length-1;x++)
			if (args[x].equals(A_FILE))
			{
				try
				{
					parameters=new ParameterDatabase(new File(new File(args[x+1]).getAbsolutePath()),args);
					break;
				}catch(FileNotFoundException e)
				{ 
					throw new RuntimeException("A File Not Found Exception was generated upon" +
						"reading the parameter file \"" + args[x+1] + "\".\nHere it is:\n" + e); 
				}catch(IOException e)
				{
					throw new RuntimeException("An IO Exception was generated upon reading the" +
						"parameter file \"" + args[x+1] +"\".\nHere it is:\n" + e);
				}
			}
		if (parameters==null)
			throw new RuntimeException("No parameter file was specified." );
		new MultiTreatmentTableMaker(parameters);
	}
}
