package sim.app.traffic;
import sim.field.network.*;
import sim.util.*;
/**
 * There is a single TrafficQueue for each incoming edge. 
 * 
 * The policy here is that at any time only cars on a given street
 * are allowed to go where ever they want.
 * 
 * @author Gabriel Balan
 */
public abstract class OneQueueGroupIntersection extends Intersection {

	public void netSetup(TrafficSim ts, Network.IndexOutIn ioi)
	{
		super.netSetup(ioi);
		int n=ioi.in.numObjs;
		int nGroups = n;
		int nQueues = n;
		allocateData(nQueues, nGroups);
		for(int i=0;i<n;i++)
		{
			TrafficQueue q = new TrafficQueue(ts, this, i, carsPassingPerFreshGreenPhasePerQueue, carsPassingPerOldGreenPhasePerQueue, takeOffTime);
			queues[i]=q;
			groups[i]=new TrafficQueue[]{q};
		}
	}
	
	public TrafficQueue getQueue(
		int incomingEdgeIndex,
		int OutgoingEdgeIndex) 
	{
		return queues[incomingEdgeIndex];
	}
	
	public Bag getQueuesForInEdge(int index, Bag b)
	{
		b.objs[0] = queues[index]; b.numObjs=1;
		return b;
	}	

//	TODO I should store the results along with the time
//	they were computed. 
//	I can also do it in an inlineable fashion.

	public long getTotalVehicles() {
		int n = queues.length;
		long sum=0;
		for(int i=0;i<n;i++)
			sum+= queues[i].totalVehicles;
		return sum;
	}

	public long getStoppedVehicles() {
		int n = queues.length;
		long sum=0;
		for(int i=0;i<n;i++)
			sum+= queues[i].stoppedVehicles;
		return sum;
	}

	public TrafficTally getDelayTally() 
	{
		//if this gets called at the end, its fine.
		//TODO if this gets called often, use a static Tally to avoid "new" ops
		int n = queues.length;
		TrafficTally tally = new TrafficTally();
		for(int i=0;i<n;i++)
			tally.add(queues[i].delayTally);
		return tally;
	}
	
//	public void updateQueueWealths()
//	{
//		int n = queues.length;
//		for(int i=0;i<n;i++)
//		{
//			TrafficQueue q = queues[i];
//			q.currentWealth = qWealthMetric.getQueueWealth(q);
//		}
//	}

//	public void setup(ParamSimState state, Parameter base) {}

	
	

}
