package sim.app.traffic;
import sim.app.traffic.info.SegmentInfo;
import sim.field.network.*;
import sim.util.*;
import ec.util.*;

/**
 *	A directed graph with Intersection as nodes;
 *	the graph is directed cause some streets might by one-way.
 *  
 * @author Gabriel Balan
 */
public abstract class RoadNet extends Network implements Parametrizable 
{
	public static final String P_THROUGH_X_DURATION= "through-intersection-time";//it does not include the waiting
	public static final String P_INTERSECTION = "intersection";

	public double intersectionCrossingDuration;
	public Intersection intersectionPrototype;
	

	public RoadNet(boolean dir){super(dir);}
	public void setup(ParamSimState state, Parameter base)
	{
		Parameter defP = defaultBase();
		Parameter interP =  base.push(P_INTERSECTION);
		intersectionPrototype  = (Intersection)state.parameters.getInstanceForParameter(
																	interP,
																	defP.push(P_INTERSECTION),
																	Intersection.class);
		intersectionPrototype.setup(state, interP);

		Parameter p;
		intersectionCrossingDuration = state.parameters.getDouble(p = base.push(P_THROUGH_X_DURATION), null, 0);
		if(intersectionCrossingDuration<0)	throw new RuntimeException("The value of "+p+" cannot be <0"); 
	}
	public Object simpleClone() {return null;}

	
	public void start()
	{
		for(int i=allNodes.numObjs-1;i>=0;--i)
			((Intersection)allNodes.objs[i]).start();
		//reset all SegmentInfo
		java.util.Iterator i = indexOutInHash.values().iterator();
		while(i.hasNext())
		{
			IndexOutIn ioi = (IndexOutIn)i.next(); 
			Bag outB = ioi.out;
			if(outB!=null)
				for(int j=outB.numObjs-1;j>=0;--j)
				{
					((SegmentInfo)((Edge)outB.objs[j]).info).start();
				}
		}	
	}
}
