package sim.app.traffic.analysis;
import sim.engine.*;
import sim.util.*;
import sim.util.intdistribution.*;
import ec.util.*;

/**
 * @author Gabriel Balan
 *
 */
public class ExpectedAloneDurationsByTripLength extends SimState
{
	double[]tripLenProbs = new double[]{6.75, 8.98, 10.47, 11.36, 11.53, 11.14, 10.21, 8.82, 6.92, 5.01, 3.54, 2.35, 1.47, 0.84, 0.42, 0.17, 0.04};
	int[]tripLengths = new int[]{1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17};

	//this should be an array of efficiencies per trip len.
	double efficiency;
	final IntDistribution tripDistrib;
	int load;

	public ExpectedAloneDurationsByTripLength(double efficiency, int load)
	{
		super(new MersenneTwisterFast(1), new Schedule());
		this.efficiency = efficiency;
		this.load = load;
		tripDistrib = new ArbitrarySupportArbitraryProbabilityDistribution(this.random, tripLenProbs, tripLengths);
		
		start();
		while(schedule.time()<1000000.0)
			if(!schedule.step(this)) break;
		finish();
	}

	public double tripDuration(int tripLength)
	{
		return 36*(tripLength+1)+ (efficiency+5)*tripLength;
	}

	Bag cars = new Bag();
	Averager avrgr;
	
	public void finish(){avrgr.printAvgs();}
	
	public void start()
	{
		for(int i=0;i<load;i++)
			cars.add(new ExpectedAloneDurationsByTripLength.Car());
		schedule.scheduleRepeating(0.0, 1, avrgr = new Averager(), 100.0);
	}
	
	class Averager implements Steppable
	{
		double distribSums[];
		int count = 0;
		public Averager()
		{
			distribSums=new double[tripLenProbs.length];
		}
		public void step(SimState sim)
		{
			for(int i = cars.numObjs-1; i>=0; i--)
			{
				// -1 cause there's no 0-length trip.
				distribSums[((Car)cars.objs[i]).tripLen -1 ]++;
			}
			count++;
		}
		
		public void printAvgs()
		{
			for(int i=0;i<distribSums.length;i++)
			{
				System.out.print((distribSums[i]/count/load)+"\t");
			}
			System.out.println();
		}
	}
	
	
	
	class Car implements Steppable
	{
		int tripLen;
		public Car(){step(ExpectedAloneDurationsByTripLength.this);}
		public void step(SimState sim)
		{
			tripLen  = tripDistrib.getInt();
			sim.schedule.scheduleOnce(sim.schedule.time()+tripDuration(tripLen), 0,ExpectedAloneDurationsByTripLength.Car.this);
		}
	}
	
	public static void main(String[] args)
	{
		new ExpectedAloneDurationsByTripLength(.250, 1000);
		new ExpectedAloneDurationsByTripLength(2.50, 1000);
		new ExpectedAloneDurationsByTripLength(25.0, 1000);
		new ExpectedAloneDurationsByTripLength(250, 1000);
	}
}
