package sim.app.traffic.analysis;

import java.io.*;

/**
 * Run this with a controller's name (e.g. "cc") as an argument

 * @author Gabriel Balan
 */
public class IntraInterContribs2Variance
{

	/**
	 * 
	 * params[0] - controller name (e.g. "cc")
	 * 
	 */
	public static void main(String[] args) throws FileNotFoundException,
			IOException
	{
		String controller = args[0];
		BufferedReader efficiencyPerTripBR = new BufferedReader(new FileReader(
				new File(controller + ".AvgDelay-byTripLen.txt")));
		BufferedReader efficiencyContribBR = new BufferedReader(new FileReader(
				new File(controller + ".Contributions2AvgDelay-byTripLen.txt")));
		BufferedReader varianceContribBR = new BufferedReader(new FileReader(
				new File(controller
						+ ".Contributions2VarianceInAvgDelay-byTripLen.txt")));

//		DecimalFormat format = new DecimalFormat("##.##########");
		int nClasses = -1;
		double njOverN[] = null;

		double intraPercentageAveragerSum = 0;
		double interPercentageAveragerSum = 0;
		
		double intraAveragerSum = 0;
		double interAveragerSum = 0;
		
		int nRuns = 0;
		while (true)
		{
			boolean exit = false;
			String line = efficiencyPerTripBR.readLine();
			if (line == null)
				break;
			nRuns++;
			String effPerTripImgs[] = line.split("\t");
			if (nClasses == -1)
			{
				nClasses = effPerTripImgs.length;
				njOverN = new double[nClasses];
			}
			if (nClasses != effPerTripImgs.length)
				throw new RuntimeException("Numbers missing in " + args[0]);

			String efficiencyContribImgs[] = efficiencyContribBR.readLine()
					.split("\t");
			if (nClasses != efficiencyContribImgs.length)
				throw new RuntimeException("Numbers missing in " + args[1]);

			String varianceContribImgs[] = varianceContribBR.readLine().split(
					"\t");
			if (nClasses != varianceContribImgs.length)
				throw new RuntimeException("Numbers missing in " + args[2]);

			// Compute [Nj/N] = efficiencyContrib_j / efficiencyPerTrip_j
			// compute global efficiency mu = average (efficiencyContrib)
			// compute variance (~fairness) = average (varianceContrib)
			// InterClass-partofVariance = Sum_j(efficiencyPerTrip_j - mu)^2
			// *[Nj/N]
			// IntraClass-partofVariance = variance - InterClass-partofVariance
			//
			// InterClass-%ofVariance = InterClass-partofVariance / Variance
			// IntraClass-%ofVariance = IntraClass-partofVariance / Variance
			// Average these across runs!!!

			double efficiencyContributionSum = 0;
			double varianceContributionSum = 0;
			double njOverNSumSanityCheck = 0;
			for (int j = 0; j < nClasses; j++)
			{
				double effContrib_j = Double
						.parseDouble(efficiencyContribImgs[j]);
				efficiencyContributionSum += effContrib_j;
				double avgPerTrip_j = Double.parseDouble(effPerTripImgs[j]);
				njOverN[j] = effContrib_j / avgPerTrip_j / nClasses;
				njOverNSumSanityCheck += njOverN[j];

				varianceContributionSum += Double
						.parseDouble(varianceContribImgs[j]);
			}
			if (Math.abs(1 - njOverNSumSanityCheck) > .000000000001)
				System.err.println("sum_j[Nj/N] SanityCheck (should be 1):\t"
						+ njOverNSumSanityCheck);
			double mu = efficiencyContributionSum / nClasses;// globalAverage
			double variance = varianceContributionSum / nClasses;

			double interClass = 0;
			for (int j = 0; j < nClasses; j++)
			{
				double mu_j = Double.parseDouble(effPerTripImgs[j]);
				double diff = mu - mu_j;
				interClass += diff * diff * njOverN[j];
			}
			double intraClass = variance - interClass;

			double interClassPerc = interClass / variance;
			double intraClassPerc = intraClass / variance;

			intraPercentageAveragerSum += intraClassPerc;
			interPercentageAveragerSum += interClassPerc;
			
			intraAveragerSum += intraClass;
			interAveragerSum += interClass;

		}

		System.out.println(controller + "\t"
				+ (intraAveragerSum / nRuns) + "\t"
				+ (intraPercentageAveragerSum / nRuns) + "\t"
				+ (interAveragerSum / nRuns) + "\t"
				+ (interPercentageAveragerSum / nRuns));

	}
}
