package sim.app.traffic.analysis;

import ec.util.MersenneTwisterFast;
import ec.util.Parameter;
import ec.util.ParameterDatabase;
import sim.app.traffic.*;
import sim.app.traffic.car.Car;
import sim.app.traffic.route.Itinerary;
import sim.engine.*;
import sim.field.network.Edge;
import sim.util.*;
import sim.app.traffic.grid.*;
import java.io.*;

/**
 * @author Gabriel Balan
 */
public class ItineraryFactoryDistributionofIntersectionsCar extends Car {

	int[][] histoCounts;
	TrafficSim tsim;
	Intersection source, destination;

	public void setup(ParamSimState state, Parameter base) 
	{
		super.setup(state, base);
		//allocate the histogram data thing
		
		tsim = (TrafficSim)state;
		GridCity grid = (GridCity)tsim.roadNet;
		
		histoCounts = new int[grid.getWidth()][grid.getHeight()];
		source = (Intersection)grid.getNode(0,0);
		destination = (Intersection) grid.getNode(grid.getWidth()-1, grid.getHeight()-1);
	}
	
	Itinerary nextItinerary()
	{
		return  tsim.itineraryFactory.getItinerary(source, destination, this);
	}
	
	public void deploy(TrafficSim tState, Intersection startingPoint, Itinerary itinerary)
	{
		currentItinerary = itinerary;
		currentIntersection = startingPoint;
		int tripLen = -1;
		//the first item in the itinerary is for the starting intersection
		//where you don't wait, so it does not count
		int nextsegIndex;
		while((nextsegIndex=itinerary.getNextSegmentIndex())!=Itinerary.NO_NEXT_SEGMENT)
		{
			tripLen++;
			Edge segment = (Edge)currentIntersection.ioi.out.objs[nextsegIndex];
			currentIntersection = (Intersection)(segment.to());
			Int2D coords = ((GridNode)currentIntersection).getCoords();
			histoCounts[coords.x][coords.y]++;
		}
				
		tState.schedule.scheduleOnceIn(1, new Steppable()
		{
			public void step(SimState s)
			{
				TrafficSim ts = (TrafficSim)s;
				ts.loadManager.destinationReached(ItineraryFactoryDistributionofIntersectionsCar.this);						
			}
		});
	}
	
	//called just once, for the prototype.
	public void finish()
	{
		for(int i=0;i<histoCounts.length;i++)
		{
			int[] histo_i = histoCounts[i];
			for(int j=0;j<histo_i.length;j++)
			{
				System.out.print(histo_i[j]+"\t");
			}
			System.out.println();
		}
	}


	public static void main(String[] args)
	{
		ParameterDatabase parameters=null;
		for(int x=0;x<args.length-1;x++)
			if (args[x].equals(MultiRun.A_FILE))
			{
				try
				{
					parameters=new ParameterDatabase(new File(new File(args[x+1]).getAbsolutePath()),args);
					break;
				}catch(FileNotFoundException e)
				{ 
					throw new RuntimeException("A File Not Found Exception was generated upon" +
						"reading the parameter file \"" + args[x+1] + "\".\nHere it is:\n" + e); 
				}catch(IOException e)
				{
					throw new RuntimeException("An IO Exception was generated upon reading the" +
						"parameter file \"" + args[x+1] +"\".\nHere it is:\n" + e);
				}
			}
		if (parameters==null)
			throw new RuntimeException("No parameter file was specified." );
			
		parameters.set(new Parameter("trafficsim.load-manager.car-generator.car"),
//						"sim.app.traffic.analysis.ItineraryFactoryDistributionofIntersectionsCar");
						"sim.app.traffic.analysis.RiggedItineraryFactoryDistributionofIntersectionsCar");
			
			
		int nRuns = 1;//parameters.getInt(MultiRun.P_RUNS, null, 1);
		if(nRuns<1) throw new RuntimeException("Too few runs.");


		double runLen = 1000;//parameters.getDouble(MultiRun.P_LENGTH, null, 0);
		if(runLen<=0) throw new RuntimeException("Too short of a run.");
		
		MersenneTwisterFast rng = new MersenneTwisterFast();
		
		for(int i=0;i<nRuns;i++)
		{
			long seed_i = rng.nextLong();
			System.out.println("\tRun "+i+"\tSeed "+seed_i);
			parameters.set(new Parameter(TrafficSim.P_SEED), seed_i+"");
			TrafficSim ts = new TrafficSim(parameters);
			Schedule schedule = ts.schedule;
			ts.start();
			while(schedule.time()<runLen)
				if (!schedule.step(ts)) break;
			ts.finish();
		}

//		//average the percentages: so far I added them up, now I divide by their number
//		for(int i=0;i<multiRunHistogram.numObjs;i++)
//		{
//			multiRunHistogram.objs[i]/=nRuns;
//		}

	}
}
