package sim.app.traffic.analysis;


import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.labels.*;
import org.jfree.data.xy.*;
import org.jfree.chart.renderer.xy.*;


import ec.util.*;
import sim.app.traffic.*;
import sim.app.traffic.queuewealth.*;
import sim.app.traffic.car.*;
import sim.engine.*;
import sim.util.*;
import sim.util.jfreechart.*;

import java.io.*;
import java.text.*;

public class WeightByLSFWrapper extends QueueWealthMetric 
{
	SumOfCarScores originalMetric;
	public static final String P_ORIG_METRIC = "original-metric";
	public void setup(ParamSimState state, Parameter base)
	{
		super.setup(state, base);
		originalMetric = (SumOfCarScores)state.parameters.getInstanceForParameter(base.push(P_ORIG_METRIC), null, SumOfCarScores.class);
		originalMetric.setup(state, base);
		//notice I'm not passing base.push(P_ORIG_METRIC)
		//cause this wrapper was sneaked in in the main.
		//(I reuse the params file!!!)
	}	

	
	public float getQueueWealth(TrafficQueue tq) 
	{
		Bag b = tq.inRangeCars.bag;
		for (int i = 0; i < b.numObjs; i++)
		{
			Car c = (Car) b.objs[i];
			recordScore(c);
		}
		return originalMetric.getQueueWealth(tq);
	}
	

	TrafficTally[] tallies = new TrafficTally[16];
	static TrafficTally[] multiRunTallies = new TrafficTally[16];
	
	
	//I record the cars' score when they enter the queue.
	//the wealth will change as they wait.
	protected void recordScore(Car c) 
	{
		int lsf = c.waitingTimeTally.count;//+1,
		//but I'd have to subtract it, to use it as index
		if(tallies.length <= lsf)
		{
			TrafficTally[] tmp = new TrafficTally[lsf+1];
			System.arraycopy(tallies, 0, tmp, 0, tallies.length);
			tallies = tmp;
		}
		TrafficTally t=tallies[lsf];
		if(t==null)
			tallies[lsf]=t=new TrafficTally();
		
		float score = originalMetric.getCarScore(c);
		t.add(score);
		
	}
	
	//it's only called from this class' main().
	public void finish()
	{
		int n = tallies.length;
		if(multiRunTallies.length<n)
		{
			TrafficTally[] tmp = new TrafficTally[n];
			System.arraycopy(multiRunTallies, 0, tmp, 0, multiRunTallies.length);
			multiRunTallies = tmp;

		}

		for(int i=0;i<n;i++)
		{
			TrafficTally t1=tallies[i];
			if(t1==null) continue;
			TrafficTally multiRunT=multiRunTallies[i];
			if(multiRunT==null)
				multiRunTallies[i]= multiRunT = new TrafficTally();
			
			multiRunT.add(t1.getAverage());
		}
	}
	


	public static void main(String[] args)
	{
		ParameterDatabase parameters=null;
		for(int x=0;x<args.length-1;x++)
			if (args[x].equals(MultiRun.A_FILE))
			{
				try
				{
					parameters=new ParameterDatabase(new File(new File(args[x+1]).getAbsolutePath()),args);
					break;
				}catch(FileNotFoundException e)
				{ 
					throw new RuntimeException("A File Not Found Exception was generated upon" +
						"reading the parameter file \"" + args[x+1] + "\".\nHere it is:\n" + e); 
				}catch(IOException e)
				{
					throw new RuntimeException("An IO Exception was generated upon reading the" +
						"parameter file \"" + args[x+1] +"\".\nHere it is:\n" + e);
				}
			}
		if (parameters==null)
			throw new RuntimeException("No parameter file was specified." );
			
		int nRuns = parameters.getInt(MultiRun.P_RUNS, null, 1);
		if(nRuns<1) throw new RuntimeException("Too few runs.");

		Parameter p = new Parameter(TrafficSim.P_ROOT).
							push(TrafficSim.P_ROAD_NET).
							push(RoadNet.P_INTERSECTION).
							push(Intersection.P_QWEALTH_METRIC);
		String originalVal = parameters.getString(p,null);
		parameters.set(p, WeightByLSFWrapper.class.getName());
		parameters.set(p.push(P_ORIG_METRIC), originalVal);

		double runLen = parameters.getDouble(MultiRun.P_LENGTH, null, 0);
		if(runLen<=0) throw new RuntimeException("Too short of a run.");
		
		MersenneTwisterFast rng = new MersenneTwisterFast();
		
		for(int i=0;i<nRuns;i++)
		{
			long seed_i = rng.nextLong();
			System.out.println("\tRun "+i+"\tSeed "+seed_i);
			parameters.set(new Parameter(TrafficSim.P_SEED), seed_i+"");
			TrafficSim ts = new TrafficSim(parameters);
			Schedule schedule = ts.schedule;
			ts.start();
			while(schedule.time()<runLen)
				if (!schedule.step(ts)) break;
			ts.finish();
			((WeightByLSFWrapper)ts.roadNet.intersectionPrototype.getQueueWealthMetric()).finish();
		}

		
		XYSeries dataSeries = new XYSeries("Weight(LSF)",false, true);
		//i provide the data in increasing X order, so:
		//autosort-false, allow duplicates-true
		System.out.print("Weight(LSF)");
		for(int i=0;i<multiRunTallies.length;i++)
		{
			TrafficTally t = multiRunTallies[i];
			if(t==null)
			{
				System.out.print("\t"+Double.NaN);
				dataSeries.add(i+1, 0);

			}
			else
			{
				double val = t.getAverage();
				System.out.print("\t"+val);
				dataSeries.add(i+1, val);
			}
		}
		System.out.println();
		
		//use this one to output the histogram
		XYSeriesCollection data = new XYSeriesCollection(dataSeries);
		
		JFreeChart chart = ChartFactory.createXYBarChart("Wealth(LSF)", "trip length so far", false, "%", data, PlotOrientation.VERTICAL, false, false, false);
		XYPlot plot = chart.getXYPlot();
		((NumberAxis)plot.getRangeAxis()).setAutoRangeIncludesZero(false);
		((NumberAxis)plot.getDomainAxis()).setTickUnit(new NumberTickUnit(1));


		XYItemRenderer itemrenderer = plot.getRenderer();
		itemrenderer.setItemLabelsVisible(true);
		itemrenderer.setItemLabelGenerator(new XYItemLabelGenerator()
		{
////			NumberFormat formatter = NumberFormat.getPercentInstance();
			DecimalFormat formatter = new DecimalFormat("#0.00");
			public String generateLabel(XYDataset dataset, int series,  int item) 
			{
				return  formatter.format(dataset.getY(series, item));
			}
		});


		PDFPrinter.printChartToPDF(chart, "weights-by-lsf.pdf");
	}
}
