package sim.app.traffic.car;

import sim.util.ParamSimState;
import ec.util.Parameter;

/**
 * This generate cars with the same max speed. 
 * <p>TODO Try a speed distribution for a more complex behavior.
 * 
 * @author Gabriel Balan
 */
public class SimpleCarGenerator implements CarGenerator 
{
	public static final String P_CAR = "car";
	public static final String P_SIMPLE_CAR_GEN = "simple-car-gen";
	public static final String P_SPEED_NUMERATOR = "speed-numerator";
	public static final String P_SPEED_DENOMINATOR = "speed-denominator";
	Car carPrototype;
	double oneOverSpeed;

		
	public Car getCar(ParamSimState state) 
	{
		Car c = (Car)carPrototype.simpleClone();
		c.oneOverSpeed = oneOverSpeed;
		return c;
	}
	
	public	void finish(){carPrototype.finish();}
	

	public void setup(ParamSimState state, Parameter base)
	{
		Parameter def = defaultBase();
		Parameter p;
		
		carPrototype = (Car)state.parameters.getInstanceForParameterEq(	p=base.push(P_CAR), 
																		def.push(P_CAR),
																		Car.class);
		carPrototype.setup(state, p);
		
		double denominator = state.parameters.getDoubleWithDefault(base.push(P_SPEED_DENOMINATOR), def.push(P_SPEED_DENOMINATOR), 1);
		if(denominator<=0) throw new RuntimeException(p+" must be >0");
		double numerator = state.parameters.getDouble(base.push(P_SPEED_NUMERATOR), def.push(P_SPEED_NUMERATOR), 0);
		if(numerator<=0) throw new RuntimeException(p+" must be>0");
		oneOverSpeed = denominator/numerator;

	}

	public Parameter defaultBase() {return new Parameter(P_SIMPLE_CAR_GEN);}
	public Object simpleClone() {return null;}
}
