package sim.app.traffic.compensation;
import sim.app.traffic.*;
import sim.util.*;
import ec.util.*;
/**
 * This determines how to compensate the cars that are waiting at red lights at 
 * a given intersection.  The amount collected from the winners is provided, so
 * 0-sum strategies can be implemented, but non-0-sum strategies are ok, too.
 * 
 * The compensation doesn't have to use the same metric as the winner determination metric
 * 
 * @author Gabriel Balan
 */
public abstract class CompensationPolicy implements Parametrizable{
	protected boolean losersOnly=true;//the default value is true.
	protected boolean stoppedOnly=false;//the default value is false.

	public static final String P_LOSERS_ONLY = "losers_only";
	public static final String P_STOPPED_ONLY = "stopped_only";
	
	
	public abstract void distribute(Intersection inter, float amount);
	
	public void setup(ParamSimState state, Parameter base)
	{
		losersOnly = state.parameters.getBoolean(base.push(P_LOSERS_ONLY), null, losersOnly);
		stoppedOnly = state.parameters.getBoolean(base.push(P_STOPPED_ONLY), null, stoppedOnly);
	}	
	public Parameter defaultBase(){return null;}
	public Object simpleClone()
	{
		try{return clone();}catch(CloneNotSupportedException ex){return null;}
	}
}
