package sim.app.traffic.compensation;
import sim.app.traffic.*;
import sim.util.*;
import ec.util.*;

/**
 * 
 * NoDeficitEqualCompensation tries to behave like FixedCompensation 
 * if there are sufficient funds.
 * Otherwise it behaves like ZeroSumEqualCompensation
 * @author Gabriel Balan
 */
public class NoDeficitEqualCompensation extends CompensationPolicy 
{
	public void distribute(Intersection inter, float amount) 
	{
		TrafficQueue[] queues = inter.queues;
		float carCount = 0;
		for(int i=0;i<queues.length;i++)
		{
			if(losersOnly&&inter.winnersMask[i]) continue;
			TrafficQueue q = queues[i];
			//TODO set the stats
			carCount+= q.inRangeCars.size();
		}
		float share  = amount/carCount;
		if(share > maxCompensation)
			share = maxCompensation;
			
		
		for(int i=0;i<queues.length;i++)
		{
			if(losersOnly&&inter.winnersMask[i]) continue;
			TrafficQueue q = queues[i];
			//TODO set the stats
			q.compensateFixedAmount(share, stoppedOnly);
		}

	}

	protected float maxCompensation;
	public static final String P_MAX_AMOUNT = "max-amount";
	public static final String P_NODEFICIT_COMPENSATION = "nodeficit-equal-compensation";

	
	public void setup(ParamSimState state, Parameter base)
	{
		super.setup(state, base);
		maxCompensation = state.parameters.getFloatWithDefault(
															base.push(P_MAX_AMOUNT), 
															base.push(FixedCompensation.P_AMOUNT),0);
	}
	public Parameter defaultBase(){return null;}
}
