package sim.app.traffic.compensation;
import sim.app.traffic.*;

/**
 * 
 * It makes sense to win based on bids, pay what you bid and get compensated
 * proportional to your bid if one starts bididng, it should pay based on that;
 * 
 * <p>Note that if one bids 0 gets 0 if its party loses. This is fair, it
 * discourages shameless free-riding.
 * Unfortunatelly, there's a BIG problem here: if you spend everything,
 * you'll never be able to get another credit.
 *
 * <p>NOTE: Make sure the bids are stricty positive cause you'll stay on 0 forever;
 * and don't bid everything on PayBid either. 
 * 
 * <p>It looks like <code>BidProportionalCompensation</code> has a few loose ends.
 * Maybe addind a "minimum compensation" (i.e. compensate proportionally
 * to the bid, but no less than 1 credit) would help?
 * 
 * <p>BIG GOOD NEWS: this method is immune to "multiple boxes" cheat ... 
 *  well, except for the counting cars tie breaking.  
 * I.e. if a driver has multiple boxes, traffic lights will treat him like multiple cars;
 * this method (together with BidSumWealthPolicy) is immune to this issue ...
 * except for ties, when counting-cars kicks in).
 * 
 * @author Gabriel Balan
 */
public class ZeroSumBidProportionalCompensation extends CompensationPolicy implements ZeroSumCompensationPolicy
{
	public void distribute(Intersection inter, float amount) 
	{
		TrafficQueue[] queues = inter.queues;
		float sum = 0;
		for(int i=0;i<queues.length;i++)
		{
			if(losersOnly&&inter.winnersMask[i]) continue;
			TrafficQueue q = queues[i];
			//TODO set the stats
			sum+=q.currentWealth;
		}
		if(sum==0) return;
		//I expect this to happen when all losing queues are empty
		float share  = amount/sum;
		for(int i=0;i<queues.length;i++)
		{
			if(losersOnly&&inter.winnersMask[i]) continue;
			TrafficQueue q = queues[i];
			q.compensateBidPropotionally(share, stoppedOnly);
		}

	}
}
