package sim.app.traffic.grid;
import sim.app.traffic.*;
import sim.app.traffic.info.SegmentInfo;
import ec.util.*;

/**
 * @author Gabriel Balan 
 **/
public class AlternatingGridCity extends GridCity 
{


	protected void connect(TrafficSim ts, Parameter base)
	{
		boolean northP,southP,eastP,westP;

		if(height%2==1 || width%2==1)
			throw new RuntimeException("In the alternating mode the grid should have even width and height, or some node will have only IN or only OUT edges.");

		northP= southP= eastP= westP = true;
		southP = westP= false;
		
		boolean northS,southS,eastS,westS;//sentinels for northP,southP,eastP and westP
		northS = northP; eastS = eastP; westS = westP; southS = southP;
		
		for(int y=0;y<height;y++)
		{
			northP = northS;southP = southS;
			for(int x=0;x<width;x++)
			{
				GridNode node = grid[y][x];
				if(y>0)
				{
					if(northP) addEdge(node, grid[y-1][x], (SegmentInfo)vSegmentInfoPrototype.simpleClone());
					if(southP) addEdge(grid[y-1][x], node, (SegmentInfo)vSegmentInfoPrototype.simpleClone());
				} 
				if(x>0) 
				{
					if(eastP) addEdge(grid[y][x-1], node, (SegmentInfo)hSegmentInfoPrototype.simpleClone());
					if(westP) addEdge(node, grid[y][x-1], (SegmentInfo)hSegmentInfoPrototype.simpleClone());
				} //nodes are swaped so the indices of nodes came naturally in the matrix
				
				northP = !(southP=!southP);
			}
			eastP = !(westP=!westP);
		}
	
		northP = northS; eastP = eastS; westP = westS; southP = southS;
		
		for(int y=0;y<height;y++)
		{
			northP = northS;southP = southS;
			for(int x=0;x<width;x++)
			{
				int indexI=0, indexO = 0;
				int wi=-1, si=-1, ei=-1, ni=-1;
				int wo=-1, so=-1, eo=-1, no=-1;
				if(y>0)
					if(northP)	no=indexO++; else ni = indexI++; 
				if(x>0)
					if(westP)	wo=indexO++; else wi = indexI++;  
				if(x<width-1)
					if(eastP)	eo=indexO++; else ei = indexI++;
				if(y<height-1)
					if(southP)	so=indexO++; else si = indexI++;

				GridNode node = grid[y][x];
				node.setPosition(x, y);
				node.netSetup(ts, (IndexOutIn)indexOutInHash.get(node), ni, no, si, so, ei, eo, wi, wo);
				node.setup(ts, base.push(P_INTERSECTION));
				northP = !(southP=!southP);
			}
			eastP = !(westP=!westP);
		}
	}

}
