package sim.app.traffic.grid;
import sim.app.traffic.*;
import sim.app.traffic.info.SegmentInfo;
import sim.util.*;
import ec.util.*;

/**
 * A <code>RoadNet</code> where the intersections (nodes) are deployed in a grid.
 * 
 * @author Gabriel Balan 
 **/
public class GridCity extends RoadNet
{
	public static final String P_WIDTH = "width";
	public static final String P_HEIGHT = "height";
	public static final String P_GRID_CITY = "grid-city";
	public static final String P_VEDGE = "vertical-edge";
	public static final String P_HEDGE = "horizontal-edge";
	public static final String V_SAME = "same";
	protected int width, height;
	public int getWidth(){return width;}
	public int getHeight(){return height;}
	protected SegmentInfo vSegmentInfoPrototype, hSegmentInfoPrototype;
	
	//TODO this must be intersection too.
	protected GridNode[][] grid;
	public GridNode getNode(int x, int y){return grid[x][y];}
	
	public GridCity(){super(true);}

	public void setup(ParamSimState state, Parameter base) 
	{
		super.setup(state, base);
		TrafficSim ts = (TrafficSim)state;
		Parameter defP = defaultBase();
		width = state.parameters.getInt(base.push(P_WIDTH), defP.push(P_WIDTH), 1);
		if(width==0) throw new RuntimeException("GridCity has 0 width.");
		height = state.parameters.getInt(base.push(P_HEIGHT), defP.push(P_HEIGHT), 1);
		if(height==0) throw new RuntimeException("GridCity has 0 height.");
		Parameter p;
		vSegmentInfoPrototype = (SegmentInfo)state.parameters.getInstanceForParameterEq(p=base.push(P_VEDGE), defP.push(P_VEDGE), SegmentInfo.class);
		vSegmentInfoPrototype.setup(state, p);

		if(state.parameters.getString(p=base.push(P_HEDGE), defP.push(P_HEDGE)).equals(V_SAME))
			hSegmentInfoPrototype = vSegmentInfoPrototype;
		else
		{
			hSegmentInfoPrototype = (SegmentInfo)state.parameters.getInstanceForParameterEq(p, defP.push(P_VEDGE), SegmentInfo.class);
			hSegmentInfoPrototype.setup(state, p);
		}
		//in grid city all vertical (horizontal) segments are the same, 
		//so I only use 2 objects to cover all Edges.
		//In a more general setup, when you expect different specializations of
		//EdgeInfo, use state.parameter.getInstace and base.push(P_EDGE).push(""+i+"-"+j) or whatever
			
		grid = new GridNode[height][width];
		if(ts.printsAllowed)System.out.println("GridCity size "+height+"x"+width);
		for(int i=0;i<height;i++)
			for(int j=0;j<width;j++)
			{
				grid[i][j]=(GridNode)intersectionPrototype.simpleClone();
			}
		// A node is first connected with the guy above then with the guy to its left
		// the other two are edges are initiated by other nodes:
		// the third edge is going to/coming from east (right) and the south connection comes last.

		connect(ts, base);
		if(ts.printsAllowed)
			printGridCity();
	}
	
	protected void printGridCity()
	{
		for(int y=0;y<height;y++)
		{
			for(int x=0;x<width;x++)
			{
				System.out.print(((Intersection)grid[y][x]).tableCellDescription(16));
			}
			System.out.println();
		}
	}
	
	protected void connect(TrafficSim ts, Parameter base)
	{ 		
		for(int y=0;y<height;y++)
		for(int x=0;x<width;x++)
		{
			GridNode node = grid[y][x];
			if(y>0)
			{
				addEdge(node, grid[y-1][x], (SegmentInfo)vSegmentInfoPrototype.simpleClone());
				addEdge(grid[y-1][x], node, (SegmentInfo)vSegmentInfoPrototype.simpleClone());
			} 
			if(x>0) 
			{
				addEdge(grid[y][x-1], node, (SegmentInfo)hSegmentInfoPrototype.simpleClone());
				addEdge(node, grid[y][x-1], (SegmentInfo)hSegmentInfoPrototype.simpleClone());
			} //nodes are swaped so the indices of nodes came naturally in the matrix
		}
		for(int y=0;y<height;y++)
		{
			int n = (y>0)?0:-1;
			for(int x=0;x<width;x++)
			{
				int index = n+1;
				int w=-1, s=-1, e=-1;
				if(x>0)			{w=index; index++;}
				if(x<width-1)	{e=index; index++;}
				if(y<height-1)	{s=index; index++;}
				GridNode node = grid[y][x];
				node.setPosition(x, y);
				node.netSetup(ts, (IndexOutIn)indexOutInHash.get(node), n, n, s, s, e, e, w, w);
				node.setup(ts, base.push(P_INTERSECTION));
			}
		}
	}
	public Parameter defaultBase() {return new Parameter(P_GRID_CITY);}
	public Object simpleClone() {return null;}
}
