package sim.app.traffic.grid;
import sim.engine.*;
import sim.field.grid.SparseGrid2D;
import sim.display.*;
import sim.display.Console;
import sim.app.traffic.*;
import sim.portrayal.grid.*;
import sim.portrayal.network.*;
import sim.app.traffic.info.*;
import sim.app.traffic.jfreechart.*;
import ec.util.*;
import java.io.*;
import java.awt.*;
import javax.swing.*;


import org.jfree.data.xy.*;
import org.jfree.chart.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.labels .*;
import org.jfree.chart.renderer.xy.*;

/**
 * @author Gabriel Balan
 */
public class GridCityTrafficSimWithUI extends GUIState
{
	public static final boolean SHOW_WEALTH_STATS = false;
	public static final boolean SHOW_DELAY_STATS = false;
	public static final boolean SHOW_QUEUE_STATS = true;
	
	
	// queues' wealths display
    public Display2D displayW;
    public JFrame displayFrameW;
    
    //queues' car-counts display 
	public Display2D displayC;
	public JFrame displayFrameC;

	//queues' average lenths display
	public Display2D displayAL;
	public JFrame displayFrameAL;

	//queues' crossing opportunities
	public Display2D displayXO;
	public JFrame displayFrameXO;


    NetworkPortrayal2D roadPortrayal = new NetworkPortrayal2D();
	SparseGridPortrayal2D qsize_intersectionPortrayal = (SHOW_QUEUE_STATS? new SparseGridPortrayal2D():null);
	SparseGridPortrayal2D qavglen_intersectionPortrayal = (SHOW_QUEUE_STATS? new SparseGridPortrayal2D():null);
	SparseGridPortrayal2D qXoppo_intersectionPortrayal = (SHOW_QUEUE_STATS? new SparseGridPortrayal2D():null);

	SparseGridPortrayal2D qwealth_intersectionPortrayal = (SHOW_WEALTH_STATS? new SparseGridPortrayal2D():null);

	WealthBoxAndWhiskerTrace aliveCarsWealthDistributionTrace = (SHOW_WEALTH_STATS? new WealthBoxAndWhiskerTrace("wealth dist trace", true):null);
	CummulativeTrafficTallyTrace delayTrace;
	AverageQueueAverageLengthTrace qlenTrace;
	TrafficColorSeqTrace uninterruptedColorSeqTrace;
	
	
    public static void main(String[] args)
    {
		ParameterDatabase parameters=null;
		for(int x=0;x<args.length-1;x++)
			if (args[x].equals(TrafficSim.A_FILE))
			{
				try
				{
					parameters=new ParameterDatabase(
							new File(new File(args[x+1]).getAbsolutePath()),
							args);
				}catch(IOException ex){ex.printStackTrace();}
				break;
			}
		if (parameters==null)
			throw new RuntimeException("No parameter file was specified." );

		TrafficSim sim = new TrafficSim(parameters);
		
		GridCityTrafficSimWithUI simWithUI = new GridCityTrafficSimWithUI(sim);
		Console c = new Console(simWithUI);
		c.setVisible(true);
	}
	

    
    public GridCityTrafficSimWithUI(SimState state) 
    {
    	super(state); 
    	TrafficSim sim = (TrafficSim)state;

    	if(aliveCarsWealthDistributionTrace!=null)
    		sim.addPhaseListener(aliveCarsWealthDistributionTrace);
    		
		if(SHOW_DELAY_STATS)
		{
			delayTrace = new DelayTrace(sim, 64);//, true, true);
    		sim.addPhaseListener(delayTrace);//I should move this into the TrafficTallyTrace
		}
		
		if(SHOW_QUEUE_STATS)
		{
			qlenTrace = new AverageQueueAverageLengthTrace(64);
			sim.addPhaseListener(qlenTrace);//I should move this into the TrafficTallyTrace
			
			uninterruptedColorSeqTrace = new TrafficColorSeqTrace(sim, 64);
			sim.addPhaseListener(uninterruptedColorSeqTrace);
		}
    }
    public static String getName(){return "Traffic Sim - Grid City";}
    
    public Object getSimulationInspectedObject() { return state; }  // non-volatile

    public void start()
	{
        super.start();        
        setupPortrayals();
	}
    
    public void load(SimState state)
    {
        super.load(state);
        setupPortrayals();
    } 
        
    public void setupPortrayals()
	{
		GridCity gridCity = (GridCity) ((TrafficSim)state).roadNet;
		GridNode[][] grid = gridCity.grid;
		SparseGrid2D map = new SparseGrid2D(gridCity.width, gridCity.height);
		for(int x=0; x<gridCity.width;x++)
			for(int y=0;y<gridCity.height;y++)
				map.setObjectLocation(grid[y][x], x, y);
        roadPortrayal.setField(new SpatialNetwork2D(map, gridCity));
		roadPortrayal.setPortrayalForAll(new SegmentPortrayal2D(true, SegmentPortrayal2D.RIGHT_HAND_SIDE_OF_ROAD));
		

		if(SHOW_WEALTH_STATS)
		{
			qwealth_intersectionPortrayal.setField(map);
			qwealth_intersectionPortrayal.setPortrayalForAll(new GridNodeWealthsPortrayal2D(.75));
	        displayW.reset();
	        displayW.repaint();
//			System.out.println("The new dataSet better be in place");		
			wealthTracePlot.setDataset(aliveCarsWealthDistributionTrace.traceDataSet);
		}
		if(SHOW_QUEUE_STATS)
		{
			qsize_intersectionPortrayal.setField(map);
			qsize_intersectionPortrayal.setPortrayalForAll(new GridNodeCountsPortrayal2D(.75));
			displayC.reset();
			displayC.repaint();
			
			qavglen_intersectionPortrayal.setField(map);
			qavglen_intersectionPortrayal.setPortrayalForAll(new GridNodeAvgQLenPortrayal2D(.75));
			displayAL.reset();
			displayAL.repaint();
			
			qXoppo_intersectionPortrayal.setField(map);
			qXoppo_intersectionPortrayal.setPortrayalForAll(new GridNodeCrossingOpportunitiesPortrayal2D(.75));
			displayXO.reset();
			displayXO.repaint();			
			

		}
    }
    
	XYPlot wealthTracePlot;
    
    public void init(Controller c)
    {
	    super.init(c);
		GridCity gridCity = (GridCity) ((TrafficSim)state).roadNet;
		int w = 500*gridCity.width/ gridCity.height;
		int h = 500;
		
		if(SHOW_QUEUE_STATS)
		{
			displayC = new Display2D(w,h,this,1);        
			displayFrameC = displayC.createFrame();
			displayFrameC.setTitle(getName() + " Count Display");
	
			c.registerFrame(displayFrameC);
			displayFrameC.setVisible(true);
			displayC.attach(roadPortrayal, "roads");
			displayC.attach(qsize_intersectionPortrayal, "count intersections");
			displayC.setBackdrop(Color.white);
			
			displayAL = new Display2D(w,h,this,1);        
			displayFrameAL = displayAL.createFrame();
			displayFrameAL.setTitle(getName() + " Average Queue Length Display");
	
			c.registerFrame(displayFrameAL);
			displayFrameAL.setVisible(true);
			displayAL.attach(roadPortrayal, "roads");
			displayAL.attach(qavglen_intersectionPortrayal, "avg length");
			displayAL.setBackdrop(Color.white);
			
			displayXO = new Display2D(w,h,this,1);        
			displayFrameXO = displayXO.createFrame();
			displayFrameXO.setTitle(getName() + " Crossing Opportunities Display");
			c.registerFrame(displayFrameXO);
			displayFrameXO.setVisible(true);
			displayXO.attach(roadPortrayal, "roads");
			displayXO.attach(qXoppo_intersectionPortrayal, "crossing  opportunities");
			displayXO.setBackdrop(Color.white);
			
			
			
			registerChart(createLineChart("Queue Average Length Trace", "Simulation time (s)", "Avg queue length", qlenTrace.dataset),
						  "Queue Length Trace",c);
						  
			registerChart(createLineChart("Uninterrupted color phases Trace", "Simulation time (s)", "Number of phases", uninterruptedColorSeqTrace.dataset),
						  "Uninterrupted color phases Trace",c);
		}
		if(SHOW_WEALTH_STATS)
		{
			displayW = new Display2D(w,h,this,1);  
	        displayFrameW = displayW.createFrame();
			displayFrameW.setTitle(getName() + " Wealth Display");
			
	        c.registerFrame(displayFrameW);
	        displayFrameW.setVisible(true);
	        displayW.attach(roadPortrayal, "roads");
			displayW.attach(qwealth_intersectionPortrayal, "wealth intersections");
	        displayW.setBackdrop(Color.white);
	        
		
			JFreeChart bAndWChart = createBoxAndWiskarChart("Wealth Distribution Trace","Time Step","Wealth");
			wealthTracePlot = bAndWChart.getXYPlot();
			((NumberAxis)wealthTracePlot.getRangeAxis()).setAutoRangeIncludesZero(false);
			registerChart(bAndWChart, "Car Wealths Trace", c);			
			
			registerChart(createHistogramChart("Wealth Distribution","Wealth","Frequency", 
										  aliveCarsWealthDistributionTrace.lastSliceDataSet),
					"Car Wealth Distribution", c);
		}		
		
		if(SHOW_DELAY_STATS)
		{
			registerChart(createLineChart(	"Overall Delay Trace", 
										"Simulation time (s)", 
										"Avg Delay time per intersection (s)", 
										delayTrace.dataset),
					 "Delay Trace", c);
		}
	}
	
	protected void registerChart(JFreeChart chart,  String title, Controller c)
	{
		((NumberAxis)chart.getXYPlot().getRangeAxis()).setAutoRangeIncludesZero(false);

		JFrame frame = new JFrame();
		frame.setResizable(true);
		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(new ChartPanel(chart), BorderLayout.CENTER);
		if(title!=null) frame.setTitle(title);
		frame.pack();
		c.registerFrame(frame);
		frame.setVisible(true);
		
	}
	
	
	protected JFreeChart createLineChart(	final String title, 
											final String xAxisCaption,
											final String yAxisCaption, 
											XYDataset xydataset)
	{
		JFreeChart jfreechart = ChartFactory.createXYLineChart(
												title, xAxisCaption, yAxisCaption,
												xydataset, PlotOrientation.VERTICAL, true, true, false);
		return jfreechart;
	}

        

	//move this to somewhere more appropriate.
	JFreeChart createHistogramChart(	final String title,
										final String xLabel,
										final String yLabel, 
										final IntervalXYDataset dataset)
	{
		JFreeChart chart = ChartFactory.createHistogram(title, xLabel, yLabel, dataset, PlotOrientation.HORIZONTAL, false, true, false);
		return chart;
	}

        
    //move this to somewhere more appropriate.
	JFreeChart createBoxAndWiskarChart(	final String title,
										final String xLabel,
										final String yLabel)
	{
		
		NumberAxis timeAxis = new NumberAxis(xLabel);		
		NumberAxis valueAxis = new NumberAxis(yLabel);
		XYBoxAndWhiskerRenderer renderer = new XYBoxAndWhiskerRenderer();//there's also a parameterless contructor.

		BoxAndWhiskerXYToolTipGenerator bwTooltipgenerator = new BoxAndWhiskerXYToolTipGenerator();
		renderer.setToolTipGenerator(bwTooltipgenerator);
		
		XYPlot plot = new XYPlot(null, timeAxis, valueAxis, renderer);
		JFreeChart chart = new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT, plot, false);//no legend

//		timeAxis.setFixedDimension(50);
		renderer.setSeriesPaint(0, java.awt.Color.red);
		return chart;
	}
        
    public void quit()
	{
		super.quit();
		
		if (displayFrameC!=null) displayFrameC.dispose();
		displayFrameC = null;  // let gc
		displayC = null;       // let gc
		
		if (displayFrameW!=null) displayFrameW.dispose();
		displayFrameW = null;  // let gc
		displayW = null;       // let gc

	}
}
