package sim.app.traffic.grid;
import sim.portrayal.*;
import java.awt.*;
import sim.app.traffic.*;
import sim.util.*;
/**
 * This works with <code>GridNode</code> objects.
 * This is based on <code>RectanglePortrayal2D</code>, which is way faster than the default Oval...
 * 
 * @author Gabriel Balan
 */
public abstract class GridNodePortrayal2D extends SimplePortrayal2D
{
	public Paint paint;
	public double scale;
	static Font labelFont = new Font("SansSerif", Font.PLAIN, 12);
	static Bag tmpBag = new Bag(4);

	public GridNodePortrayal2D() { this(Color.gray,.5); }
	public GridNodePortrayal2D(Paint paint) { this(paint,.5); }
	public GridNodePortrayal2D(double scale) { this(Color.gray,scale); }
    
	public GridNodePortrayal2D(Paint paint, double scale)
	{
		this.paint = paint;
		this.scale = scale;
	}
                
	/** If drawing area intersects selected area, add last portrayed object to the bag */
	public boolean hitObject(Object object, DrawInfo2D range)
		{
		final double width = range.draw.width*scale;
		final double height = range.draw.height*scale;
		return( range.clip.intersects( range.draw.x-width/2, range.draw.y-height/2, width, height ) );
		}

	// assumes the graphics already has its color set
	public void draw(Object object, Graphics2D graphics, DrawInfo2D info)
		{
		final double width = info.draw.width*scale;
		final double height = info.draw.height*scale;

		graphics.setPaint(paint);
		// we are doing a simple draw, so we ignore the info.clip

		final int x = (int)(info.draw.x - width / 2.0);
		final int y = (int)(info.draw.y - height / 2.0);
		final int w = (int)(width);
		final int h = (int)(height);

		// draw centered on the origin
		graphics.fillRect(x,y,w,h);
		
		//so far this was copied from RectanglePortrayal2D. Now printing the queue sizes.
		graphics.setFont(labelFont);

		FontMetrics fm = graphics.getFontMetrics();
		GridNode gn = (GridNode)object;
		int[] inEdgeIndices = gn.getInEdgeIndices();
		int[] outEdgeIndices = gn.getOutEdgeIndices();
		Intersection i = (Intersection)object;
		int index;
		int padding = 5;
		int yPadding =padding + fm.getDescent();
		
		boolean showInRangeStats = showInRangeStats();
		
		if((index = inEdgeIndices[GridNode.N])!=-1)
		{
			Bag queues = i.getQueuesForInEdge(index, tmpBag);
			int wOverN = w/(queues.numObjs+1);
			if(showInRangeStats)
				print(graphics, x+w+wOverN, y-yPadding, -wOverN, 0, CENTERED, queues, GridNode.N, outEdgeIndices, i, INRANGE_OUTQ_STAT);
			print(graphics, x+w+wOverN, y+yPadding, -wOverN, 0, CENTERED, queues, GridNode.N, outEdgeIndices, i, QUEUE_STAT);
		}
		
		if((index=inEdgeIndices[GridNode.E])!=-1)
		{
			Bag queues = i.getQueuesForInEdge(index, tmpBag);
			int hOverN = h/(queues.numObjs+1);
			print(graphics, x+w-padding, y+h+hOverN, 0, -hOverN, RIGHT, queues, GridNode.E, outEdgeIndices, i, QUEUE_STAT);
			if(showInRangeStats)
				print(graphics, x+w+padding, y+h+hOverN, 0, -hOverN, LEFT, queues, GridNode.E, outEdgeIndices, i, INRANGE_OUTQ_STAT);
		}

		if((index=inEdgeIndices[GridNode.S])!=-1)
		{
			Bag queues = i.getQueuesForInEdge(index, tmpBag);
			int wOverN = w/(queues.numObjs+1);
			print(graphics, x, y+w-yPadding, wOverN, 0, CENTERED, queues, GridNode.S, outEdgeIndices, i, QUEUE_STAT);
			if(showInRangeStats)
				print(graphics, x, y+w+yPadding, wOverN, 0, CENTERED, queues, GridNode.S, outEdgeIndices, i, INRANGE_OUTQ_STAT);
		}
		
		if((index=inEdgeIndices[GridNode.W])!=-1)
		{
			Bag queues = i.getQueuesForInEdge(index, tmpBag);
			int hOverN = h/(queues.numObjs+1);
			print(graphics, x+padding, y, 0, hOverN, LEFT, queues, GridNode.W, outEdgeIndices, i, QUEUE_STAT);
			if(showInRangeStats)
				print(graphics, x-padding, y, 0, hOverN, RIGHT, queues, GridNode.W, outEdgeIndices, i, INRANGE_OUTQ_STAT);
		}

	}
	
	protected final boolean QUEUE_STAT = true;
	protected final boolean INRANGE_OUTQ_STAT = false;
	
	protected boolean showInRangeStats(){return false;}
	protected String getInRangeStatText(TrafficQueue q){return null;}

	protected abstract String getQueueStatText(TrafficQueue q);
		
	static final float LEFT = 0;
	static final float CENTERED = 0.5f;
	static final float RIGHT = 1;
	
	static final Color[] trafficColors = new Color[]{Color.red, Color.yellow, Color.green};

	//TODO the hit part of hitOrDraw is not covered        
    void print(	Graphics2D graphics, 
    			int x, int y, int  dx, int dy,
    			float xJust, 
    			Bag queues, int startIndex, int[] outEdgeIndices, Intersection inter, boolean qDesc)
    {
		FontMetrics fm = graphics.getFontMetrics();
		int hdx = dx/2;
		int hdy = dy/2;
		x+=hdx;
		y+=hdy;
		int yoffset = hdy -fm.getHeight()/2+fm.getAscent();
    	for(int i=0;i<4;i++)
    	{
    		int index = outEdgeIndices[(i+startIndex)%4];
    		if(index==-1) continue;
    		TrafficQueue q =(TrafficQueue)queues.objs[index];
    		
			graphics.setPaint(trafficColors[q.color]);

    		//draw q in the square x, y, x+dx, y+dy.
    		int minx = (dx>0)? x:(x+dx);
			int miny = (dy>0)? y:(y+dy);
			
    		String s = (qDesc==QUEUE_STAT)?getQueueStatText(q):getInRangeStatText(q);
			int wText = fm.stringWidth(s);
			graphics.drawString(s, minx+hdx-xJust*wText, miny+yoffset);
    		x+=dx;
    		y+=dy;
    	}
    }
}
