package sim.app.traffic.grid;

import sim.app.traffic.*;
import sim.app.traffic.car.Car;
import sim.app.traffic.route.*;
import sim.util.Int2D;
import sim.field.network.*;
import ec.util.*;
/**
 * 
 * The itinerary could be any shortest path.  Since we're talking about a grid,
 * this is equivalent to generating any path inside the bounding box 
 * containing the source and destination, with the restriction 
 * that it has the "minimum number of blocks."
 * Expected intersection waiting times are of no concern here.
 * 
 * <p>At each node where there's a choice (go vertical or horizontal),
 * this class flips a 50-50 coin. This does not produce a uniform
 * distribution over itineraries. I already used this method in 
 * a paper, so I will keep this class for result replicability/
 * backward compatibility, although it is not that complicated to 
 * achieve the uniform distribution for grids (see <code>
 * UniformRandomMinBlockItineraryFactory</code>)
 *  (in general you'd need a Dynamic Programming-like approach).
 * 
 * @author Gabriel Balan
 *
 */
public class SimpleRandomMinBlockItineraryFactory extends ItineraryFactory 
{
	public static final String P_RND_MINBLOCK_ITINERARY_FACTORY = "rnd-minBlock-itinerary-factory";
	public Parameter defaultBase(){return new Parameter(P_RND_MINBLOCK_ITINERARY_FACTORY);}
	static class MutableInt {public int value;}
	static final MutableInt xBlocks = new MutableInt();
	static final MutableInt yBlocks = new MutableInt();

	public Itinerary getItinerary(
		Intersection source,
		Intersection destination,
		Car car) 
	{
		GridNode currentNode;
		Int2D startingPoint = (currentNode = (GridNode)source).getCoords();
		Int2D destinationPoint = ((GridNode)destination).getCoords();
		xBlocks.value =destinationPoint.x - startingPoint.x;
		yBlocks.value =destinationPoint.y - startingPoint.y;
		int xDirection, yDirection;


		//W is towards the smaller column indices
		if(xBlocks.value>=0)
			xDirection = GridNode.E;
		else
		{
			xDirection = GridNode.W;
			xBlocks.value = -xBlocks.value; 
		}
		
		//N is towards smaller row indices
		if(yBlocks.value>=0)
			yDirection = GridNode.S;
		else
		{
			yDirection = GridNode.N;
			yBlocks.value = -yBlocks.value; 
		}	
		int nblocks = xBlocks.value+yBlocks.value;
		int[] data = new int[nblocks];
		for(int i=0;i<nblocks;i++)
		{

			int direction;
			MutableInt dirBlocks;
			if(		(xBlocks.value>0 && yBlocks.value>0 && state.random.nextBoolean())//choose randomly between going x or y
				||	(xBlocks.value>0 && yBlocks.value==0))
			{
				direction = xDirection;
				dirBlocks = xBlocks;
			}
			else
			{
				direction = yDirection;
				dirBlocks = yBlocks;
			}
			int outIndex = data[i] = currentNode.getOutEdgeIndices()[direction];
			dirBlocks.value--;
			Edge edge = (Edge)currentNode.getIOI().out.objs[outIndex];
			currentNode = (GridNode)edge.getOtherNode(currentNode);
		}
		return new FixedItinerary(data);
	}
}
