package sim.app.traffic.grid.torus;
import sim.app.traffic.*;
import sim.app.traffic.grid.*;
import sim.app.traffic.info.SegmentInfo;
import sim.field.network.Edge;
import sim.util.*;
import ec.util.*;

/**
 * @author Gabriel Balan 
 **/
public class AlternatingTorusCity extends GridCity 
{


	protected void connect(TrafficSim ts, Parameter base)
	{
		boolean northP,southP,eastP,westP;

		northP= southP= eastP= westP = true;
		southP = westP= false;
		
		boolean northS,southS,eastS,westS;//sentinels for northP,southP,eastP and westP
		northS = northP; eastS = eastP; westS = westP; southS = southP;
		
		for(int y=0, ny=height-1; y<height; y++, ny=y-1)
		{
			//equivalent with "int ny = (y+height-1)%height"
			northP = northS;southP = southS;
			for(int x=0, nx=width-1;  x<width;  x++, nx=x-1)
			{
				GridNode node = grid[y][x];
				node.setPosition(x, y);
				if(northP) addEdge(node, grid[ny][x], (SegmentInfo)vSegmentInfoPrototype.simpleClone());
				if(southP) addEdge(grid[ny][x], node, (SegmentInfo)vSegmentInfoPrototype.simpleClone());
				if(eastP) addEdge(grid[y][nx], node, (SegmentInfo)hSegmentInfoPrototype.simpleClone());
				if(westP) addEdge(node, grid[y][nx], (SegmentInfo)hSegmentInfoPrototype.simpleClone());
				
				northP = !(southP=!southP);
			}
			eastP = !(westP=!westP);
		}
	
		northP = northS; eastP = eastS; westP = westS; southP = southS;
		
		for(int y=0;y<height;y++)
		{
			for(int x=0;x<width;x++)
			{
				
				GridNode node = grid[y][x];

				//other than the "2 edges between 2 nodes have the same index in the IN and OUT" 
				//the code is not very smart: I make no effort to find the formula for the order of 
				// the edges in the bags; I simply ask
				
				int wo=-1, so=-1, eo=-1, no=-1;
				Bag outEdgeBag = getEdgesOut(node);
				for(int i=0;i<2;i++)//there better be 2.
				{
					Edge edge = (Edge)outEdgeBag.objs[i];
					Int2D neighborCoords = ((GridNode)edge.to()).getCoords();
					int dx = (x-neighborCoords.x+width)%width;
					int dy = (y-neighborCoords.y+height)%height; 
					if(dx==0)
						if(dy==1)
							no=i;
						else
							so=i;
					else
						if(dx==1)
							wo=i;
						else
							eo=i;
				}
				
				int wi=-1, si=-1, ei=-1, ni=-1;
				Bag inEdgeBag = getEdgesIn(node);
				for(int i=0;i<2;i++)//they're better be 2.
				{
					Edge edge = (Edge)inEdgeBag.objs[i];
					Int2D neighborCoords = ((GridNode)edge.from()).getCoords();
					int dx = (x-neighborCoords.x+width)%width;
					int dy = (y-neighborCoords.y+height)%height; 
					if(dx==0)
						if(dy==1)
							ni=i;
						else
							si=i;
					else
						if(dx==1)
							wi=i;
						else
							ei=i;
				}
				
				node.netSetup(ts, (IndexOutIn)indexOutInHash.get(node), ni, no, si, so, ei, eo, wi, wo);
				node.setup(ts, base.push(P_INTERSECTION));
				northP = !(southP=!southP);
			}
			eastP = !(westP=!westP);
		}
	}

}
