package sim.app.traffic.grid.torus;

import sim.app.traffic.*;
import sim.app.traffic.grid.*;
import sim.app.traffic.car.Car;
import sim.app.traffic.route.*;
import sim.util.Int2D;
import sim.field.network.*;
import ec.util.*;
import sim.util.*;
/**
 * The itinerary could be any path inside the rectangle defined by 
 * the four coordinates (the source and destination).
 * 
 * <p>The restriction is "minimum number of blocks"!
 * Expected intersection waiting times are of no concern here.
 * 
 * <p>Fewest turns (1 turn tops, since we're in the grid scenario).
 * There are three variations:
 * <ol>
 * <li> randomly choose which dimension to travel first.
 * <li> shorter leg first
 * <li> longer leg first
 * </ol>
 *  @author Gabriel Balan
 */
public class OneTurnTopsItineraryFactory4Torus extends ItineraryFactory 
{
	public static final String P_1TURNTOPS_ITINERARY_FACTORY = "1turntops-itinerary-factory-4torus";
	public static final String P_MODE = "mode";
	public static final String V_SF = "smaller-leg-first";
	public static final String V_LF = "bigger-leg-first";
	public static final String V_XF = "X-leg-first";
	public static final String V_YF = "Y-leg-first";
	public static final String V_RF = "random";
	
	public static final int SF = 0;
	public static final int LF = 1;
	public static final int RF = 2;
	public static final int XF = 3;
	public static final int YF = 4;
	int mode;
	GridCity city;
	
	public Parameter defaultBase(){return new Parameter(P_1TURNTOPS_ITINERARY_FACTORY);}
	public void setup(ParamSimState state, Parameter base)
	{
		super.setup(state, base);
		String smode = state.parameters.getStringWithDefault(base.push(P_MODE), defaultBase().push(P_MODE), V_RF);
		if(smode.equals(V_SF)) mode = SF;
		else
			if(smode.equals(V_LF)) mode = LF;
		else
			if(smode.equals(V_RF)) mode = RF;
		else
			if(smode.equals(V_XF)) mode = XF;
		else
			if(smode.equals(V_YF)) mode = YF;
		else
			throw new RuntimeException("Invalid mode "+smode);
			
		city = (GridCity)((TrafficSim)state).roadNet;
	}

	public Itinerary getItinerary(
		Intersection source,
		Intersection destination,
		Car car) 
	{
		GridNode currentNode;
		Int2D startingPoint = (currentNode = (GridNode)source).getCoords();
		Int2D destinationPoint = ((GridNode)destination).getCoords();
		
		
		int xBlocks= directedBlocks(destinationPoint.x, startingPoint.x, city.getWidth());
		int yBlocks= directedBlocks(destinationPoint.y, startingPoint.y, city.getHeight());
		
		int xDirection, yDirection;
		
		
		//assume (0,0) to be the upper left corner.

		//W is towards the smaller column indices
		if(xBlocks>=0)
			xDirection = GridNode.E;
		else
		{
			xDirection = GridNode.W;
			xBlocks= -xBlocks; 
		}
		
		//N is towards smaller row indices
		if(yBlocks>=0)
			yDirection = GridNode.S;
		else
		{
			yDirection = GridNode.N;
			yBlocks = -yBlocks; 
		}	
		int nblocks = xBlocks + yBlocks;
		int[] data = new int[nblocks];

		
		//I will go x, y and x again.
		//in each of the modes one of these three will be 0.
		int firstXLegBlocks=0;
		switch(mode)
		{
			case SF: 	if(xBlocks<yBlocks || (xBlocks==yBlocks && state.random.nextBoolean()))//ties broken randomly
						{
							firstXLegBlocks = xBlocks;
							xBlocks = 0;
						}
						else firstXLegBlocks = 0;
						break;
			case LF: 	if(xBlocks>yBlocks || (xBlocks==yBlocks && state.random.nextBoolean()))
						{
							firstXLegBlocks = xBlocks;
							xBlocks = 0;
						}
						else firstXLegBlocks = 0;
						break;
			case RF: 	if(state.random.nextBoolean())
						{
							firstXLegBlocks = xBlocks;
							xBlocks = 0;
						}
						else firstXLegBlocks = 0;
						break;
			case XF:	firstXLegBlocks = xBlocks;
						xBlocks=0;
						break;
			case YF:	break;//nothing to do, firstXLegBlocks is already 0.
		}
		
		int index = 0;
		for(int i=0;i<firstXLegBlocks;i++)
		{
			int outIndex = data[index++] = currentNode.getOutEdgeIndices()[xDirection];
			Edge edge = (Edge)currentNode.getIOI().out.objs[outIndex];
			currentNode = (GridNode)edge.getOtherNode(currentNode);
		}
		for(int i=0;i<yBlocks;i++)
		{
			int outIndex = data[index++] = currentNode.getOutEdgeIndices()[yDirection];
			Edge edge = (Edge)currentNode.getIOI().out.objs[outIndex];
			currentNode = (GridNode)edge.getOtherNode(currentNode);
		}
		for(int i=0;i<xBlocks;i++)
		{
			int outIndex = data[index++] = currentNode.getOutEdgeIndices()[xDirection];
			Edge edge = (Edge)currentNode.getIOI().out.objs[outIndex];
			currentNode = (GridNode)edge.getOtherNode(currentNode);
		}
		return new FixedItinerary(data);
	}
		
	/**
	 * for the x direction, a positive value means EAST; for the y direction, it means SOUTH
	 */
	public int directedBlocks(final int coordD, final int coordS, int length)
	{
		int diff = coordD-coordS;
		int sign = +1;
		int distNoWraps = diff;
		if(diff<0)
		{
			sign = -1;
			distNoWraps = -diff;
		}
		int distAround = length -distNoWraps;
		if(distNoWraps<distAround)
			return diff;
		if(distNoWraps>distAround)
			return distAround * (-sign);
		//else (distNoWraps==distAround)
		//so we flip a coin to decide which way to go.
		if(state.random.nextBoolean())
			return distNoWraps;
		else
			return -distNoWraps;
	}
}
