package sim.app.traffic.grid.torus;

import ec.util.*;
import sim.app.traffic.*;
import sim.app.traffic.car.*;
import sim.app.traffic.grid.*;
import sim.app.traffic.load.*;
import sim.app.traffic.route.*;
import sim.field.network.Edge;
import sim.util.ParamSimState;
import sim.util.intdistribution.IntDistribution;

/**
 * Constant Model with cars driving randomly for a fixed number of blocks.
 * 
 * <p>Use this with a torus grid city.
 * 
 * @author Gabriel Balan
 * 
 */
public class RandomWalkContantLoadMananger extends ConstantLoadManager
{
	public static final String P_TRIP_LENGTH = "trip-length";
	public static final String P_UTURNS = "allow-Uturns";

	public static final String P_RND_WALK = "rnd-walk_ct-load-manager";

	boolean uturnAllowed=false;
	
	public Parameter defaultBase()
	{
		return new Parameter(P_RND_WALK);
	}

	IntDistribution lenDistrib;


	public void setup(ParamSimState state, Parameter base)
	{
		super.setup(state, base);
		Parameter p;
		lenDistrib = (IntDistribution)state.parameters.getInstanceForParameter(
				p=base.push(P_TRIP_LENGTH),
				defaultBase().push(P_TRIP_LENGTH), 
				IntDistribution.class);
		lenDistrib.setup(state, p);
		uturnAllowed = state.parameters.getBoolean(base.push(P_UTURNS), defaultBase().push(P_UTURNS), uturnAllowed);
	}

	
	public void destinationReached(Car car)
	{
		if(resetWealthModuloTrips>0 && car.destinationsReached % resetWealthModuloTrips == 0)
				car.resetWealthAndWaitingTimes(tSim, tSim.initialWealth);
		canCar(car);
		spawnACar();
	}

	protected void spawnACar()
	{
		Car c = carGenerator.getCar(tSim);
		aliveCars.add(c);
		c.destinationsReached = 0;
		c.wealth = tSim.initialWealth;

		RoadNet net = tSim.roadNet;

		int sourceIndex = odSelector.getRandomSourceIndex(tSim.random);
		Intersection source = (Intersection) net.allNodes.objs[sourceIndex];
		GridNode currentNode = (GridNode) source;

		int tripLen = lenDistrib.getInt();
		int[] data = new int[tripLen];
		
		if(uturnAllowed)
			for (int i = 0; i < tripLen; i++)
			{
				int outIndex = data[i] = currentNode.getOutEdgeIndices()[tSim.random.nextInt(4)];
				Edge edge = (Edge) currentNode.getIOI().out.objs[outIndex];
				currentNode = (GridNode) edge.getOtherNode(currentNode);
			}
		else
		{
			int oldDirection = -1;
			for (int i = 0; i < tripLen; i++)
			{
				int outIndex = data[i] = currentNode.getOutEdgeIndices()[oldDirection=nextDirection(oldDirection)];
				Edge edge = (Edge) currentNode.getIOI().out.objs[outIndex];
				currentNode = (GridNode) edge.getOtherNode(currentNode);
			}
		}		
		c.deploy(tSim, source, new FixedItinerary(data));
	}
	
	

	private int nextDirection(int oldDirection)
	{
		if(oldDirection==-1)
			return tSim.random.nextInt(4);
		
//		static final int N = 0;
//		static final int E = 1;
//		static final int S = 2;
//		static final int W = 3;	
		int back = (oldDirection+2)%4;
			
		//imagine that the last direction replaces BACKdirection 
		//so I have a compact set of 3 directions to choose from.
		int direction = tSim.random.nextInt(3);
		//remember, the last one replaced the tabu one. 
		if(direction==back) 
			return 3;
		return direction;
	}
}
