package sim.app.traffic.grid.torus;

import ec.util.Parameter;
import sim.app.traffic.Intersection;
import sim.app.traffic.RoadNet;
import sim.app.traffic.car.*;
import sim.app.traffic.grid.GridNode;
import sim.app.traffic.route.*;
import sim.field.network.Edge;
import sim.util.ParamSimState;
import sim.util.intdistribution.ArbitrarySupportArbitraryProbabilityDistribution;
/**
 * Constant Model with cars driving straight 
 * (randomly choosing the direction) for
 * a fixed number of blocks.
 * 
 * <p>Use this with a torus grid city.
 * 
 * @author Gabriel Balan
 *
 */
public class StraightWalkContantLoadMananger extends RandomWalkContantLoadMananger
{
	public static final String P_PROB = "prob";
	public static final String P_DRIVE_STRAIGHT = "straight-walk_load-manager";
	public Parameter defaultBase()
	{
		return new Parameter(P_DRIVE_STRAIGHT);
	}
	
	//Random direciton generator;
	ArbitrarySupportArbitraryProbabilityDistribution rdg;
	
	
	
	public void setup(ParamSimState state, Parameter base)
	{
		super.setup(state, base);
		
		Parameter prob = base.push(P_PROB);
		double[] probs = new double[4];
		for(int i=0;i<4;i++)
		{
			probs[i]=state.parameters.getDoubleWithDefault(prob.push(GridNode.directionName[i]), null, 1);
		}
		rdg = new ArbitrarySupportArbitraryProbabilityDistribution(tSim.random, probs);
		//ArbitrarySupportArbitraryProbabilityDistribution has a setup() function too
		//but I want "prob.N" in the param file, not "prob.0"
		
	}
	
	



	protected void spawnACar()
	{
		Car c = carGenerator.getCar(tSim);
		aliveCars.add(c);
		c.destinationsReached = 0;
		c.wealth = tSim.initialWealth;

		RoadNet net = tSim.roadNet;

		int sourceIndex = odSelector.getRandomSourceIndex(tSim.random);
		Intersection source = (Intersection) net.allNodes.objs[sourceIndex];
		GridNode currentNode = (GridNode)source;
		
		int tripLen = lenDistrib.getInt();
		int[] data = new int [tripLen];
		int direction = rdg.getInt();
		for(int i=0;i<tripLen;i++)
		{
			int outIndex = data[i] = currentNode.getOutEdgeIndices()[direction];
			Edge edge = (Edge)currentNode.getIOI().out.objs[outIndex];
			currentNode = (GridNode)edge.getOtherNode(currentNode);
		}
		c.deploy(tSim, source, new FixedItinerary(data));
	}
}
