package sim.app.traffic.grid.torus;
import sim.app.traffic.*;
import sim.app.traffic.grid.*;
import sim.app.traffic.info.*;
import ec.util.Parameter;

/**
 * 
 * @author Gabriel Balan
 *
 */
public class TorusCity extends GridCity
{
	
	private void addEdge(	GridNode source, 		int[] sourceIndices, 		int sourceIndexIndex,
							GridNode destination,	int[] destinationIndices,	int destinationIndexIndex)
	{
		addEdge(source, destination,  (SegmentInfo)vSegmentInfoPrototype.simpleClone());
		destinationIndices[destinationIndexIndex]=destination.getIOI().in.numObjs-1;
		sourceIndices[sourceIndexIndex]=source.getIOI().out.numObjs-1;
		//it's not going to work if source==destination && !directed
	}
	
	
	protected void connect(TrafficSim ts, Parameter base)
	{ 		
		int[][][] indices = new int[height][width][8];
		final int NIN_INDEX = 0;
		final int NOUT_INDEX = 1;
		final int SIN_INDEX = 2;
		final int SOUT_INDEX = 3;
		final int EIN_INDEX = 4;
		final int EOUT_INDEX = 5;
		final int WIN_INDEX = 6;
		final int WOUT_INDEX = 7;
				
		//just so the intersections come in a nice order (not really necessary).
		for(int y=0; y<height; y++)
			for(int x=0; x<width;  x++)
			{
				GridNode node = grid[y][x];
				addNode(node);
				((Intersection)node).netSetup((IndexOutIn)indexOutInHash.get(node));
			}
		
		for(int y=0, ny=height-1; y<height; y++, ny=y-1)
		for(int x=0, nx=width-1;  x<width;  x++, nx=x-1)
		{
			//equivalent with "int ny = (y+height-1)%height"
			GridNode node = grid[y][x];
			node.setPosition(x, y);
			int[] nodeIndices = indices[y][x];

			
			GridNode otherNode = grid[ny][x];
			int[] otherNodeIndices = indices[ny][x];
			addEdge(node, nodeIndices, NOUT_INDEX, otherNode, otherNodeIndices, SIN_INDEX);
			addEdge(otherNode, otherNodeIndices, SOUT_INDEX, node, nodeIndices, NIN_INDEX);
			
			otherNode = grid[y][nx];
			otherNodeIndices = indices[y][nx];
			addEdge(node, nodeIndices, WOUT_INDEX, otherNode, otherNodeIndices, EIN_INDEX);
			addEdge(otherNode, otherNodeIndices, EOUT_INDEX, node, nodeIndices, WIN_INDEX);

		}
		for(int y=0;y<height;y++)
		{
			for(int x=0;x<width;x++)
			{
				
				GridNode node = grid[y][x];
				int[] nodeIndices = indices[y][x];
				node.netSetup(	ts, (IndexOutIn)indexOutInHash.get(node), 
								nodeIndices[NIN_INDEX], nodeIndices[NOUT_INDEX], 
								nodeIndices[SIN_INDEX], nodeIndices[SOUT_INDEX],
								nodeIndices[EIN_INDEX], nodeIndices[EOUT_INDEX],
								nodeIndices[WIN_INDEX], nodeIndices[WOUT_INDEX]);
				node.setup(ts, base.push(P_INTERSECTION));
			}
		}
	}
	
	
	
}
