package sim.app.traffic.info;
import sim.portrayal.*;
import java.awt.*;
import sim.field.network.*;
import sim.portrayal.network.*;


/**
 * 
 * Given two nodes A and B, the edges AB and BA will have their 
 * labels on top of each other if you use SimpleEdgePortrayal2D.
 * This offsets the labels based on the direction of the edge:
 * 					 |i j k|
 * offset ~ k x SE = |0 0 1|, where SE = END-START = [x,y]T
 * 					 |x y 0|
 * 
 * Of course, if your graph is undirected, stick to SimpleEdgePortrayal2D.
 * 
 * @author Gabriel Balan
 */
public class SegmentPortrayal2D extends SimpleEdgePortrayal2D
{
	final int dir;//+/-1
	public static final boolean RIGHT_HAND_SIDE_OF_ROAD = true;
	public static final boolean LEFT_HAND_SIDE_OF_ROAD = false;
    
	public SegmentPortrayal2D(boolean label, boolean sideOfRoad)
	{
		super(Color.black, Color.black, label? Color.black: null);
		dir = sideOfRoad==RIGHT_HAND_SIDE_OF_ROAD?1:-1;
	}
    
	public void draw(Object object, Graphics2D graphics, DrawInfo2D info)
		{
		if (!(info instanceof EdgeDrawInfo2D))
			throw new RuntimeException("Expected this to be an EdgeDrawInfo2D: " + info);
		EdgeDrawInfo2D e = (EdgeDrawInfo2D) info;
        
		final int startX = (int)e.draw.x;
		final int startY = (int)e.draw.y;
		final int endX = (int)e.secondPoint.x;
		final int endY = (int)e.secondPoint.y;
		final int midX = (int)((e.draw.x+e.secondPoint.x) / 2);
		final int midY = (int)((e.draw.y+e.secondPoint.y) / 2);
        
		// draw lines
		if (fromPaint == toPaint)
			{
			graphics.setPaint (fromPaint);
			graphics.drawLine (startX, startY, endX, endY);
			}
		else
			{
			graphics.setPaint( fromPaint );
			graphics.drawLine(startX,startY,midX,midY);
			graphics.setPaint( toPaint );
			graphics.drawLine(midX,midY,endX,endY);
			}
            
		// draw label
		if (labelPaint != null)
			{
			//Object infoval = ((Edge)object).info;
			String information = getLabel((Edge)object, e);
			if( /* infoval != null && */ information.length() > 0 )
				{
				graphics.setPaint(labelPaint);
				graphics.setFont(labelFont);
				FontMetrics fm = graphics.getFontMetrics(); 
				int width = fm.stringWidth(information);
				
				int hw = width/2;
				int extraPadding = fm.getLeading()+3;//not sure about the leaging, cause it's 0.
				int dx = dir*(endX -startX);
				graphics.drawString( information, 
									 midX - hw + (hw+extraPadding)*sign(startY-endY),
									 midY + (dx==0? fm.getAscent()-fm.getHeight()/2:
									 			(dx>0? 	 fm.getAscent() +extraPadding :
									  					-fm.getDescent()-extraPadding)));
				//drawString(.., x, y): (x, y)=baseline of the first character
                
				}
			}
		}
	int sign(int x){return (x==0)? 0: (x<0?-dir:dir);}
}
