package sim.app.traffic.jfreechart;

import sim.app.traffic.*;
import org.jfree.data.xy.*;

/**
 * @author Gabriel Balan
 *
 */
public abstract class CummulativeTrafficTallyTrace implements TrafficSim.InfoDisplayUpdater
{
	final int modulo;
	final TrafficTally[] mainTallies;
	final TrafficTally[] oldTallies;
	final TrafficTally diffTally;	
	final XYSeries[] series;
	public XYSeriesCollection dataset = new XYSeriesCollection();
	protected TrafficSim tSim;
	final boolean doStdevs;

	public CummulativeTrafficTallyTrace(TrafficSim tsim, int modulo, int nTallies)
	{
		this(tsim, modulo, nTallies, true);
	}


	public CummulativeTrafficTallyTrace(TrafficSim tsim, int modulo, int nTallies, boolean doStdevs)
	{
		this.tSim = tsim;
		this.doStdevs = doStdevs;
		int seriesPerTally = doStdevs? 4:2;
		
		int n = nTallies;
		mainTallies = new TrafficTally[n];
		oldTallies = new TrafficTally[n];
		diffTally = new TrafficTally();
		series  = new XYSeries[seriesPerTally*n];
		String s =" ("+modulo+" phases)";

		updateTallies(mainTallies);
		
		boolean autoSort = false;
		//No thanx, I'll give JFreeChart the data already sorted.		
		boolean allowDuplicates = true;
		//I don't actually plan to have duplicates, 
		//but I don't want JFreeChart to get even slower by activelly preventing them.

		for(int i=0;i<n;i++)
		{
			oldTallies[i]=new TrafficTally();
			
			String name = mainTallies[i].getName();
			series[i*seriesPerTally+0] = new XYSeries(name+"-Global Avg", autoSort, allowDuplicates);
			series[i*seriesPerTally+1] = new XYSeries(name+"-Local Avg "+s, autoSort, allowDuplicates);
			if(doStdevs)
			{
				series[i*seriesPerTally+2] = new XYSeries(name+"-Global StDev", autoSort, allowDuplicates);
				series[i*seriesPerTally+3] = new XYSeries(name+"-Local Stdev"+s, autoSort, allowDuplicates);
			}
		}
		
		for(int i=0;i<series.length;i++)dataset.addSeries(series[i]);
		this.modulo = modulo;
	}
	
	public abstract void updateTallies(TrafficTally[] tallies);
	
	public void reset() {for(int i=0;i<series.length;i++) series[i].clear();}
	
	int index = 0;	
	public void add(TrafficSim sim) 
	{		
		index++;
		if(index%modulo!=0) return;
		index = 0;
		double t = sim.schedule.time();
		if(t<2*sim.roadNet.intersectionPrototype.phaseDuration+sim.warmupPeriod)
			return;
			
		updateTallies(mainTallies);
		int n = mainTallies.length;
		for(int i=0;i<n;i++)
		{
			TrafficTally tally = mainTallies[i];
			series[i*4+0].add(t, tally.getAverage());
			if(doStdevs)
				series[i*4+2].add(t, tally.getStDev());
			
			
			diffTally.minus(tally, oldTallies[i]);
			oldTallies[i].set(tally);
			
			series[i*4+1].add(t, diffTally.getAverage());
			if(doStdevs)
				series[i*4+3].add(t, diffTally.getStDev());
		}
	}
}
