package sim.app.traffic.load;

import sim.app.traffic.TrafficSim;
import sim.util.*;
import ec.util.*;
/**
 * ORIGIN-DESTINATION selector
 * the default values are 1 for every cell but [i,i] which is 0.
 * 
 * @author Gabriel Balan
 */
public class FullOD extends ODSelector 
{
	public static final String P_OD = "OD";
	double[][] cummultiveProbs;
	double[] cummulativeSourceProbs;

	boolean uniformSourceDist;
	boolean[] uniformDestinationDists;
	
	public Parameter defaultBase(){return new Parameter(P_OD);}
	public void setup(ParamSimState state, Parameter base)
	{
		TrafficSim ts = (TrafficSim)state;
		Parameter def = defaultBase();
		int n = ts.roadNet.allNodes.numObjs;
		cummultiveProbs = new double[n][n];
		cummulativeSourceProbs = new double[n];
		uniformDestinationDists = new boolean[n];

		double sum = 0;
		double pij, p0=0;
		uniformSourceDist = true;
		for(int i=0;i<n;i++)
		{
			double rowSum=0;
			double pi0=0;
			boolean uniformRow_i_Distrib = true;
			double[] cummultiveProbsRow = cummultiveProbs[i];
			for(int j=0;j<n;j++)
			{
				pij= state.parameters.getDoubleWithDefault(base.push(i+"").push(j+""), 
																def.push(i+"").push(j+""),((i==j)?0:1));
				rowSum+=pij;
				if(j==0)
					pi0 = pij;
				else
					if(uniformRow_i_Distrib && pij!=pi0) uniformRow_i_Distrib=false;
			}
			cummultiveProbsRow[0]/=rowSum;
			for(int j=1;j<n;j++)
				cummultiveProbsRow[j] = cummulativeSourceProbs[j-1] + cummulativeSourceProbs[j]/rowSum;
			uniformDestinationDists[i] = uniformRow_i_Distrib;

			cummulativeSourceProbs[i] = rowSum;						
			sum+=rowSum;
			if(i==0)
				p0 = rowSum;
			else
				if(uniformSourceDist && rowSum!=p0) uniformSourceDist=false;			
		}
		
		cummulativeSourceProbs[0]/=sum;
		for(int i=1;i<n;i++)
			cummulativeSourceProbs[i] = cummulativeSourceProbs[i-1] + cummulativeSourceProbs[i]/sum;

	}
	
	
	
	public int getRandomSourceIndex(MersenneTwisterFast rng)
	{
		int n = cummulativeSourceProbs.length;
		if(uniformSourceDist)
			return rng.nextInt(n);
		else
			return getRandomIndex(rng.nextDouble(), cummulativeSourceProbs, 0, n-1);
	}

	public int getRandomDestinationIndex(MersenneTwisterFast rng, int sourceIndex)
	{
		int n = cummulativeSourceProbs.length;
		if(uniformDestinationDists[sourceIndex])
			return rng.nextInt(n);
		else
		{
			double[] cummulativeDestProbs = cummultiveProbs[sourceIndex];
			return getRandomIndex(rng.nextDouble(), cummulativeDestProbs, 0, n-1);
		}
	}
}
