package sim.app.traffic.load;
import sim.app.traffic.*;
import ec.util.*;
import sim.util.*;
/**
 * ORIGIN-DESTINATION selector with probs of choosing 
 * a source and a destination being independent
 * (aside from a destination cannot be the same as the source)
 *
 * @author Gabriel Balan
 */
public class IndependentOD extends ODSelector {
	public static final String P_IOD = "INDEPENDENT_OD";
	public static final String P_SOURCE_PROB = "source-probability";
	public static final String P_DEST_PROB = "destination-probability";

	public Parameter defaultBase(){return new Parameter(P_IOD);}

	protected double[] cummulativeSourceProbs;
	protected double[] cummulativeDestProbs;
	boolean uniformSourceDistrib;
	boolean uniformDestDistrib;

	public void setup(ParamSimState state, Parameter base)
	{
		TrafficSim ts = (TrafficSim)state;
		int n = ts.roadNet.allNodes.numObjs;
		cummulativeDestProbs = new double[n];
		cummulativeSourceProbs = new double[n];
		double sSum = 0, dSum = 0;
		Parameter def = defaultBase();
		
		double dp0=0,sp0=0; 
		double spi, dpi;
		uniformDestDistrib = true;
		uniformSourceDistrib =true;
		for(int i=0;i<n;i++)
		{
			Parameter base_i = base.push(""+i);
			Parameter def_i = def.push(""+i);
			dpi = state.parameters.getDoubleWithDefault(base_i.push(P_DEST_PROB), 
														 def_i.push(P_DEST_PROB), 1);
			spi = state.parameters.getDoubleWithDefault(base_i.push(P_SOURCE_PROB), 
														 def_i.push(P_SOURCE_PROB), 1);
																			 
			dSum += cummulativeDestProbs[i] = dpi;
			sSum += cummulativeSourceProbs[i] = spi;
			
			if(i==0)
			{
				dp0 = dpi;
				sp0 = spi;
			}
			else
			{
				if(uniformDestDistrib && dpi!=dp0) uniformDestDistrib=false;
				if(uniformSourceDistrib && spi!=sp0) uniformSourceDistrib=false;
			}
		}
		
		//I could precompute 1/dSum and 1/sSum, since * is cheaper than /, but the whole fn is only called once, so ...
		cummulativeDestProbs[0]/=dSum;
		cummulativeSourceProbs[0]/=sSum;
		for(int i=1;i<n;i++)
		{
			cummulativeDestProbs[i] = cummulativeDestProbs[i-1] + cummulativeDestProbs[i]/dSum;
			cummulativeSourceProbs[i] = cummulativeSourceProbs[i-1] + cummulativeSourceProbs[i]/sSum;
		}
	}
	
	public int getRandomSourceIndex(MersenneTwisterFast rng)
	{
		int n = cummulativeSourceProbs.length;
		if(uniformSourceDistrib)
			return rng.nextInt(n);
		else
			return getRandomIndex(rng.nextDouble(), cummulativeSourceProbs, 0, n-1);
	}
	public int getRandomDestinationIndex(MersenneTwisterFast rng, int tabooIndex)
	{
		int n_1 = cummulativeDestProbs.length-1;
		if(uniformDestDistrib)
		{// I should temporarily switch the last index with tabooIndex
		 // and draw one of first n-1 indices.
			int index = rng.nextInt(n_1);
			if(index == tabooIndex)
				index = n_1;
			return index;
		} 
		else
		{
			double cummulativeTabooProb = cummulativeDestProbs[tabooIndex];
			double cummulativeLeftToTaboo =(tabooIndex==0)? 0 : cummulativeDestProbs[tabooIndex-1];
			double tabooValue = cummulativeTabooProb - cummulativeLeftToTaboo;
			if(tabooValue == 0 )
				return getRandomIndex(rng.nextDouble(), cummulativeDestProbs, 0, n_1);
				
//			double cummulativeRightToTaboo = 1d - cummulativeTabooProb;
			//I now choose between left and right side of tabooIndex
			double rn = rng.nextDouble() * (1-tabooValue);
			if(rn<=cummulativeLeftToTaboo)
				return getRandomIndex(rn, cummulativeDestProbs, 0, tabooIndex-1);
			else
				return getRandomIndex(rn+tabooValue, cummulativeDestProbs, tabooIndex+1, n_1);
			
		}
	}
}
