package sim.app.traffic.load;
import ec.util.*;
import sim.util.*;
import sim.app.traffic.car.*;
import sim.util.doubledistribution.*;
import sim.engine.*;
import sim.app.traffic.*;
/**
 * <ul>
 * <li>There are no frozen cars or sleeping cars.
 * <li>When a car reaches its destination, it dies off. 
 * </ul>
 * 
 * <p>Cars are inserted in the system according to the Poisson Arrival Model
 * (i.e. exponential inter-arrival times)
 * 
 * <p>Others use a Poisson Arrival Model for each source link.
 * What are the differences when I have a single injection distribution
 * and then choose the source according to the source probability of each intersection???
 * 
 * <p>TODO Should the mean be divided by the number of intersections? 
 * the number of bound intersections? (this would corespond to external links)
 * 
 * @author Gabriel Balan
 * 
 */
public class RandomODTrafficLoadManager extends TrafficLoadManager
{

	public static final String P_RANDOMOD_MANAGER = "random-OD-traffic-load-manager";
	public Parameter defaultBase(){return new Parameter(P_RANDOMOD_MANAGER);}
	public static final String P_IAD = "interarrival-distribution";
	DoubleDistribution interArrivalTimeDist;
	public static final String P_INITIAL_LOAD = "initial-load";
	int initialLoad=0;
	
	public void setup(ParamSimState state, Parameter base)
	{
		super.setup(state, base);
		Parameter base_iad, base_iad_seed;
		interArrivalTimeDist = (DoubleDistribution) state.parameters
				.getInstanceForParameter(base_iad = base.push(P_IAD),
						defaultBase().push(P_IAD), DoubleDistribution.class);

		base_iad_seed = base_iad.push(DoubleDistribution.P_SEED);
		if (!state.parameters.exists(base_iad_seed))
			state.parameters.set(base_iad_seed, state.random.nextInt() + "");
		interArrivalTimeDist.setup(state, base_iad);
		
		initialLoad = state.parameters.getIntWithDefault(base.push(P_INITIAL_LOAD), 
														 defaultBase().push(P_INITIAL_LOAD),
														 initialLoad);
	}
	
	public void start()
	{
		super.start();
		for(int i=0;i<initialLoad;i++) spawnACar();
		//scheduleSpawning();//I cant schedule before 0
		//so here goes:
		tSim.schedule.scheduleOnce(interArrivalTimeDist.getDouble(),
				TrafficSim.CAR_ORDERING,
				spawCarAndRewindTimer);
		//THIS IS NOT JUST LIKE IN "scheduleSpawning()"
		//scheduleOnce vs. scheduleOnceInDelta!!!
	}
	
	private void scheduleSpawning()
	{
		tSim.schedule.scheduleOnceIn(interArrivalTimeDist.getDouble(), 
													spawCarAndRewindTimer,
													TrafficSim.CAR_ORDERING);
	}
	Steppable spawCarAndRewindTimer = new Steppable(){public void step(SimState state){spawnACar();scheduleSpawning();}};
	
	public void destinationReached(Car car)
	{
		if(resetWealthModuloTrips>0 && car.destinationsReached % resetWealthModuloTrips == 0)
				car.resetWealthAndWaitingTimes(tSim, tSim.initialWealth);
		canCar(car);
	}
}