package sim.app.traffic.queuewealth;

import sim.app.traffic.*;
import sim.util.*;
import ec.util.*;

/**
 * This returns a weighted sum of multiple wealth metrics.
 * 
 * @author Gabriel Balan
 */
public class MultiQueueWealthWeightedSum extends QueueWealthMetric 
{
	QueueWealthMetric[] components;
	float[] weights;
	public static final String P_COUNT = "count";
	public static final String P_WEIGHT = "weight";
	
	
	public void setup(ParamSimState state, Parameter base)
	{
		int count = state.parameters.getInt(base.push(P_COUNT),null);
		if(count<=0)
			throw new RuntimeException("Two few components");
		
		components  = new QueueWealthMetric[count];
		weights = new float[count];
		for(int i=0;i<count;i++)
		{
			Parameter p = base.push(""+i);
			QueueWealthMetric ci = (QueueWealthMetric)state.parameters.getInstanceForParameter(p, null, QueueWealthMetric.class);
			ci.setup(state, p);
			components[i] = ci;
			weights[i]=state.parameters.getFloatWithDefault(p.push(P_WEIGHT), null, 1);
		}
	}

	public float getQueueWealth(TrafficQueue tq) 
	{
		float sum = 0;
		for(int i=components.length-1;i>=0;i--)
			sum+=components[i].getQueueWealth(tq)*weights[i];
		return sum;
	}
}
