package sim.app.traffic.queuewealth;
import sim.app.traffic.*;
import sim.app.traffic.car.*;
import sim.util.*;
import ec.util.*;
/**
 * @author Gabriel Balan
 *
 */
public class SumOfCarAvgWaitingTimes extends SumOfCarScores 
{
	//if a car has no delay samples, what's the AWT?
	// 0 makes sense, 
	//but so does 1, cause I bring some CC into the process
	//(assuming the delays are going to be bigger than 1)
	//[I know I'd be adding number of cars with seconds here]
	double defaultAWTvalue = 0;
	TrafficSim tstate;
	static public final String P_DEFAULT_AWT_VALUE = "default-AWT-value";
	static public final String P_AWTS_SUM = "AWTs-sum";
	static public final String V_SIM_AWT = "SIMULATION_AWT";
	static public final String V_AWT_NOW = "AWT+CURRENT";

	static final int AWT_WITH_VALUE_DEFAULT = 0;
	static final int AWT_WITH_SIM_DEFAULT = 1;
	static final int AWT_WITH_CURRENT = 2;

	int  mode = AWT_WITH_VALUE_DEFAULT;
		
	public Parameter defaultBase(){return new Parameter(P_AWTS_SUM);}
	public void setup(ParamSimState state, Parameter base)
	{
		tstate = (TrafficSim)state;
		String modeImg = state.parameters.getStringWithDefault(base.push(P_DEFAULT_AWT_VALUE), defaultBase().push(P_DEFAULT_AWT_VALUE), ""+defaultAWTvalue);
		if(modeImg.equals(V_SIM_AWT))
			mode = AWT_WITH_SIM_DEFAULT;
		else
		if(modeImg.equals(V_AWT_NOW))
			mode = AWT_WITH_CURRENT;
		else
			defaultAWTvalue = state.parameters.getDoubleWithDefault(base.push(P_DEFAULT_AWT_VALUE),
																defaultBase().push(P_DEFAULT_AWT_VALUE),
																defaultAWTvalue);
	}	
	
	/**
	 * I do the memoization in here because there's only one QueueWealthMetric instance 
	 * in the whole system (the intersections get it from the prototypical intersection).
	 * IF THE INTERSECTIONS BECOME HETEROGENEOUS, the memoization should move to TrafficSim
	 */
	public float getQueueWealth(TrafficQueue tq) 
	{
		double now;
		switch(mode)
		{	
			case AWT_WITH_CURRENT:
				now=tstate.schedule.time();
				return tq.getSumOfCarsAvgWTPlusCurrent(now);
				
				
			case AWT_WITH_SIM_DEFAULT:
				if((now=tstate.schedule.time())>tallyBirthday)
				{
					defaultAWTvalue =	tstate.getOverallDelayTally().getAverageWithDefault(0);
					tallyBirthday = now;
				}
				//NO BREAK
			case AWT_WITH_VALUE_DEFAULT:
				return tq.getSumOfCarsAvgWT(defaultAWTvalue);
			default:
				throw new RuntimeException("invalid mode "+mode);
		}
	}
	
	
	double tallyBirthday = -1;

	public float getCarScore(Car c)
	{
//		TrafficQueue tq;
		double now;
		switch(mode)
		{	
			case AWT_WITH_CURRENT:
				now=tstate.schedule.time();
				return (float)c.waitingTimeTally.getAverageWithExtraValue(c.getTimeInCurrentQueue(now));
				
			case AWT_WITH_SIM_DEFAULT:
				if((now=tstate.schedule.time())>tallyBirthday)
				{
					defaultAWTvalue =	tstate.getOverallDelayTally().getAverageWithDefault(0);
					tallyBirthday = now;
				}
				//NO BREAK
			case AWT_WITH_VALUE_DEFAULT:
				return (float)c.waitingTimeTally.getAverageWithDefault(defaultAWTvalue);
			default:
				throw new RuntimeException("invalid mode "+mode);
		}
		
	}
	
}
