package sim.app.traffic.queuewealth.delayincrease;
import sim.app.traffic.*;
import sim.app.traffic.car.Car;
import sim.util.*;

/**
 * @author Gabriel Balan
 * 
 * Enhanced Counting Cars:
 * _20_ secs 
 * 		for all cars stopped
 * _20-T_ 
 * 		for all moving cars that could reach the intersection in T &lt 20 
 * 		and would otherwise go through
 * _0_
 * 		for all moving cars that wouln't make it through anyways
 * 		(0 crossing opportunities)
 * 
 * Differences from DelayIncrease_v2:
 * -no no more takeoff penalties for stopped cars; I use a fixed 20secs flat penalty 
 */
public class DelayIncrease_v3 extends DelayIncrease_v2 
{
	/**
	 * My intersection is considering giving me the RED/GREEN light. How much
	 * bigger is my delay at this intersection going to be if i get RED instead of GREEN?
	 */
	public float getQueueWealth(TrafficQueue tq) 
	{
		float totalExtraDelay = 0;
		Bag inRangeCarBag = tq.inRangeCars.bag;
		int numberInRangeCars = inRangeCarBag.numObjs;
		if(numberInRangeCars==0)
			return totalExtraDelay;
			

		double nextPhaseDuration = tq.owner.phaseDuration;
		// /upper limit on the number of cars I'll be able to
		// get through next phase (assuming I win)
		int crossingOpportunitiesNextPhase = (tq.color == TrafficQueue.GREEN) ? 
				tq.getMaxCarsPassingPerOldGreenPhase():
				tq.getMaxCarsPassingPerFreshGreenPhase();
		int qSize = tq.size();
		java.util.Iterator iter = tq.iterator();
		while(iter.hasNext())
		{
			Car c = (Car)iter.next();
			double delay_i = nextPhaseDuration;
			if (crossingOpportunitiesNextPhase > 0)
				crossingOpportunitiesNextPhase--;
			totalExtraDelay += (fn==null?delay_i:fn.delayContributuion(c, delay_i));
		}

		double now = tq.state.schedule.time();
		int numberInRangeMovingCars = inRangeCarBag.numObjs-qSize;
		if (numberInRangeMovingCars != 0 && crossingOpportunitiesNextPhase != 0)
		{
			if(numberInRangeMovingCars <=crossingOpportunitiesNextPhase)
				//all cars in range could pass through.
				for (int i = 0; i < numberInRangeCars; i++)
				{
					Car c = (Car) inRangeCarBag.objs[i];
					if(c.isStoppedInQueue()) continue;
					double delay_i = Math.max(nextPhaseDuration-c.getTimeTillIntersection(now), 0);
					totalExtraDelay += (fn==null?delay_i:fn.delayContributuion(c, delay_i));
				}
			else
			{
				// only the first "crossingOpportunitiesNextPhase" cars
				// would be affected by the next RED/GREEN decision

				Double[] keys = tmpHeapKeys;
				Car[] values = tmpHeapValues;
				if(keys.length<numberInRangeCars)
				{//resize
					keys = new Double[numberInRangeCars];
					values = new Car[numberInRangeCars];
				}

				int index = 0;
				for (int i = 0; i < numberInRangeCars; i++)
				{
					Car c = (Car) inRangeCarBag.objs[i];
					if(c.isStoppedInQueue()) continue;
					values[index] = c;
					keys[index] = new Double(c.getTimeTillIntersection(now));
					index++;
				}
				Heap h = new Heap(keys, values, numberInRangeMovingCars);
				for (int x = 0; x < crossingOpportunitiesNextPhase; x++)
				{
					double timeTillIntersection = ((Double)h.getMinKey()).doubleValue();
					Car c = (Car)h.extractMin();
					double delay_i = Math.max(nextPhaseDuration	- timeTillIntersection, 0);
					totalExtraDelay += (fn==null?delay_i:fn.delayContributuion(c, delay_i));
				}
			}
		}
		if(totalExtraDelay<0)
			throw new RuntimeException("Error: DelayIncrease<0");
		return totalExtraDelay;
	}	
}
