package sim.app.traffic.queuewealth.delayincrease;
import sim.app.traffic.*;
import sim.app.traffic.car.*;
import sim.app.traffic.load.*;
import sim.app.traffic.queuewealth.CountingCars;
import sim.app.traffic.analysis.*;
import sim.util.*;
import ec.util.*;
/**
 * 1. I need the distribution of trip lengths to compute the _expectedTL(LSF)_
 * 		see AvgTLofLSF.computeExpected_TL_ofLSF()
 * 2. I get the distribution of trip lengths by running a special main(String[]),
 * that runs TripLenDistribAnalyzerCar.main(String[]) and 
 * TrafficSim.main(String[]) OR is it MultiRun.main(String[])
 * 
 * TripLenDistribAnalyzerCar prints out stuff, so scripts that retrive stuff from logs 
 * won't work!
 * 
 * @author Gabriel Balan
 */
public class ExpectedTripLenNormalization implements
		ExtraDelayContributionFunction {
	public static final String P_EXPECTEDTL = "expectedTL-normalization";	
	public static final String P_TL_DISTRIB = "trip-len-distrib";
	public static final String P_MAX_TRIP_LEN = "max-trip-len";
	double[] tldistrib;
	double[] expectedTLValues;
	public void setup(ParamSimState state, Parameter base)
	{
		Parameter def = defaultBase();
		int n=state.parameters.getInt(base.push(P_MAX_TRIP_LEN), def.push(P_MAX_TRIP_LEN));
		tldistrib = new double[n];
		
		Parameter p = base.push(P_TL_DISTRIB);
		Parameter defp= def.push(P_TL_DISTRIB);
		for(int i=1;i<=n;i++)		//1-based!!!
		{
			tldistrib[i-1]=state.parameters.getDouble(p.push(""+i),defp.push(""+i),0);
			if(tldistrib[i-1]<0)
				throw new RuntimeException("Invalid value for\n"+
											p.push(""+i)+"\n"+defp.push(""+i));
		}
		
		expectedTLValues = AvgTLofLSF.computeExpected_TL_ofLSF(tldistrib, null);
	}	
	
	public double delayContributuion(Car c, double extraDelay) 
	{
		return extraDelay/expectedTLValues[c.waitingTimeTally.count/*+1-1*/];
		//+1 cause if you have k entries it means you got past k intersections, and this is the kth+1.
		//-1 cuase there's no LSF=0 and all cells are shifted to the left one position.
	}
	
	public static void main(String[] args)
	{
		int argc = args.length;
		//let's run TripLenDistribAnalyzerCar.
		String[] phase1Args = new String[argc+2*8];
		System.arraycopy(args, 0, phase1Args, 0, argc);
		phase1Args[argc]="-p";		
		phase1Args[argc+1]=new Parameter(new String[]{TrafficSim.P_ROOT,TrafficSim.P_LOAD_MANAGER, TrafficLoadManager.P_CAR_GEN})+
									"="+SimpleCarGenerator.class.getName();
		phase1Args[argc+2]="-p";		
		phase1Args[argc+3]=new Parameter(new String[]{TrafficSim.P_ROOT,TrafficSim.P_LOAD_MANAGER, TrafficLoadManager.P_CAR_GEN, SimpleCarGenerator.P_CAR})+
									"="+TripLenDistribAnalyzerCar.class.getName();
		phase1Args[argc+4]="-p";		
		phase1Args[argc+5]=MultiRun.P_RUNS+"=20";
		phase1Args[argc+6]="-p";		
		phase1Args[argc+7]=MultiRun.P_LENGTH+"=5000";
		phase1Args[argc+8]="-p";		
		phase1Args[argc+9]=new Parameter(new String[]{TrafficSim.P_ROOT,TrafficSim.P_LOAD_MANAGER, ConstantLoadManager.P_LOAD})+
									"=1";
		//I sure hope we're using a ConstantLoadManager.
		//(an open model will just work a tone more time).
		//(I can't specify it here, cause it can be randomChain_Constload, random_ConstLoad, etc.


//		trafficsim.road-net.intersection.queue-wealth-metric = sim.app.traffic.queuewealth.CountingCars
		phase1Args[argc+10]="-p";		
		phase1Args[argc+11]=new Parameter(new String[]{	TrafficSim.P_ROOT,
														TrafficSim.P_ROAD_NET, 
														RoadNet.P_INTERSECTION,
														Intersection.P_QWEALTH_METRIC})+
									"="+CountingCars.class.getName();
		phase1Args[argc+12]="-p";
		phase1Args[argc+13]=TrafficSim.P_NO_PRINTS+"=true";
		phase1Args[argc+14]="-p";		
		phase1Args[argc+15]=ParameterDatabase.PRINT_PARAMS+"=false";
		
		

		DoubleBag tlDistribution=TripLenDistribAnalyzerCar.getTripLenDistribution(phase1Args);
		
		
		String[] phase2Args = new String[argc+2*tlDistribution.numObjs+2];
		System.arraycopy(args, 0, phase2Args, 0, argc);
		
		phase2Args[argc]="-p";
		phase2Args[argc+1]= new Parameter(new String[]{P_EXPECTEDTL, P_MAX_TRIP_LEN,})+"="+tlDistribution.numObjs;
		
		for(int i=0;i<tlDistribution.numObjs;i++)
		{
			phase2Args[argc+2*i+2]="-p";		
			phase2Args[argc+2*i+3]=new Parameter(new String[]{P_EXPECTEDTL, P_TL_DISTRIB,""+(i+1)})+"="+tlDistribution.objs[i];
		}
		MultiRun.main(phase2Args);
	}
	public Parameter defaultBase(){return new Parameter(P_EXPECTEDTL);}
	public Object simpleClone(){return null;}
}
