package sim.app.traffic.rotation;
import sim.app.traffic.*;
import sim.util.*;

/**
 * If all queues are empty, return the lastWinningGroupIndex.
 * 
 * Otherwise behave as RoundRobin, except it skips the queue groups with no cars
 * 
 * 
 * <p>I wrote this because I want not to deal with uturns when the routes are fixed before the trip; 
 * HOWEVER, same as RR, it might work better with OneQueueGroupIntersection (one group per incoming edge)
 * than with OneLaneOutEdgeIntersection, where groups share queues.
 * 
 * @author Gabriel Balan
 */
public class NonEmptyRoundRobin implements RotationPolicy {
	IntBag queueCarCounts=  new IntBag();
	IntBag groupCarCounts=  new IntBag();
	
	public int getWinningGroupIndex(TrafficSim simState, Intersection i) 
	{
		int lastWinningGroupIndex = i.lastWinningGroupIndex;
		int winningGroupCandidateIndex = lastWinningGroupIndex;
		do
		{
			winningGroupCandidateIndex = (winningGroupCandidateIndex+1)%i.groups.length;
			TrafficQueue[] group_composition = i.groups[winningGroupCandidateIndex];
			for(int j=0;j<group_composition.length;j++)
			{
				TrafficQueue q = group_composition[j];
				if( q.inRangeCars.size()!=0) return winningGroupCandidateIndex;
			}
		}while(winningGroupCandidateIndex!=lastWinningGroupIndex);
		
		return winningGroupCandidateIndex;
	}
}
