package sim.app.traffic.rotation;
import sim.app.traffic.*;
import sim.util.*;
/**
 * @author Gabriel Balan
 *
 */
public class WealthiestFirst implements RotationPolicy 
{
	static IntBag ties = new IntBag();
	public int getWinningGroupIndex(TrafficSim simState, Intersection inter) 
	{
		//I have the queues memoize the queueWealth values 
		inter.updateQueueWealths();		
		float[] groupWealths = inter.groupWealths;
		int nGroups = inter.groups.length;
//		int wealthiestGroupIndex = -1;
		float biggestTotalWealth = -1;
		for(int i=0;i<nGroups;i++)
		{
			float w_group_i=0;
			TrafficQueue[] group_i = inter.groups[i];
			for(int j=0;j<group_i.length;j++)
				w_group_i+= group_i[j].currentWealth;
			groupWealths[i]=w_group_i;
			if(w_group_i>biggestTotalWealth)
			{
				biggestTotalWealth = w_group_i;
//				wealthiestGroupIndex = i;
				ties.clear(); ties.add(i);
			}
			else
			if(w_group_i==biggestTotalWealth) ties.add(i);
			
		}
		
		//if I'll ever model the LOST TIME, the one with green last time should have priority.
		
		if(ties.numObjs==1) return ties.objs[0];//don't waste time waking up the RNG

		if(ties.numObjs==nGroups && biggestTotalWealth==0)//either every car is flat-broke or the intersection is empty.
			return mostCarsGroup(simState, inter);
		return ties.objs[simState.random.nextInt(ties.numObjs)];
	}
	
	//when all cars are flat-broke, I fall back to COUNTING CARS
	protected int mostCarsGroup(TrafficSim simState, Intersection inter)
	{
		int nGroups = inter.groups.length;
		float biggestCrowd = -1;

		for(int i=0;i<nGroups;i++)
		{
			float count_group_i=0;
			TrafficQueue[] group_i = inter.groups[i];

			for(int j=0;j<group_i.length;j++)
				count_group_i+= group_i[j].inRangeCars.size();
			if(count_group_i > biggestCrowd)
			{
				biggestCrowd = count_group_i;
				ties.clear(); ties.add(i);
			}
			else
			if(count_group_i==biggestCrowd) ties.add(i);	
		}
		if(ties.numObjs==1) return ties.objs[0];//don't waste time waking up the RNG
		if(biggestCrowd==0)	return inter.lastWinningGroupIndex;//if the intersection is empty, I won't switch the lights
		return ties.objs[simState.random.nextInt(ties.numObjs)];
	}

}
