package sim.app.traffic.route;
import sim.util.*;
/**
 * @author Gabriel Balan
 *  
 * Features a precomputed itinerary. No later-on updates are considered
 */
public class FixedItinerary implements Itinerary 
{
	int[] segmentIndices;
	int nextSegmentIndexIndex;
	int maxSegmentIndexIndex;
	
	public FixedItinerary(int[] data)
	{
		segmentIndices = data;
		nextSegmentIndexIndex = 0;
		maxSegmentIndexIndex = segmentIndices.length;
//		System.out.print("Itinerary:");
//		for(int i=0;i<data.length;i++)System.out.print(data[i]+" ");
//		System.out.println();		

	}
	
	public FixedItinerary(IntBag data)
	{
		segmentIndices = data.objs;
		nextSegmentIndexIndex = 0;
		maxSegmentIndexIndex = data.numObjs;
	}
		
	public int remainingSegments()
	{
		return maxSegmentIndexIndex - nextSegmentIndexIndex;
	}
	
	/**  
	 * @returns the index of the next segment in the out Bag or 
	 * NO_NEXT_SEGMENT if there is no next segment (you just finished the last one)
	 **/
	public int getNextSegmentIndex()
	{
		if(nextSegmentIndexIndex>= maxSegmentIndexIndex)
			return NO_NEXT_SEGMENT;
		else
			return segmentIndices[nextSegmentIndexIndex++];	
//		//smaller byte-code than 
//		return (nextSegmentIndexIndex>=maxSegmentIndexIndex)?
//					NO_NEXT_SEGMENT:
//					segmentIndices[nextSegmentIndexIndex++];	
	}
	
	public int getNumberIntersesctions2Cross()
	{
		return maxSegmentIndexIndex-1;
		//-1 cause the first element in the itinerary
		//is how you get out of the "source" intersection
		//which you don't cross
	}
	
	public boolean lastSegment()
	{
		return (nextSegmentIndexIndex >= maxSegmentIndexIndex);
	}
}
