package sim.app.traffic.route;

/**
 * @author Gabriel Balan
 *  
 * Theoretically a driver could replan after every segment under perfect information 
 * (a part of an optimal path is optimal). However, this would be very slow 
 * especially if the replanning never changes anything (it is based on
 * static topological information).
 * 
 * If I do "all pairs shortest paths", replaning means reading one entry in the matrix. 
 * One problem here would be that you cannot model different drivers having different
 * information about the traffic (and learning based on their own experiences). The 
 * second issue is how ofter do you recompute this.
 * 
 * If everybody is using the same (static) planner, you don't get the 
 * "100 take the best road, 10 take the longer road" effect.
 *
 */
public interface Itinerary 
{
	/**  
	 * @returns the index of the next segment in the out Bag or 
	 * NO_NEXT_SEGMENT if there is no next segment (you just finished the last one)
	 * 
	 * TODO if the itinerary is to be updated, make sure it's updated inside this call
	 **/
	int getNextSegmentIndex();	
	final static int NO_NEXT_SEGMENT=-1;
	
	/**
	 * @returns whether we're on the last segment of the trip.
	 * 
	 * XXX i'm not sure this would be appropriate for all implementations.
	 * Alternativelly, I could employ a pushBackLastSegmentIndex().
	 */
	boolean lastSegment();
}
