package sim.app.traffic.route;
import sim.app.traffic.*;
import sim.app.traffic.car.*;
import sim.util.ParamSimState;
import ec.util.Parameter;

/**
 * @author Gabriel Balan
 *
 * Discrete distribution.
 */
public class RndMuxItineraryFactory extends ItineraryFactory 
{
	public static final String P_MUX_ITIN_FACT = "mux-itinerary-factory";
	public static final String P_COUNT = "count";
	public static final String P_PROB = "prob";
	
	ItineraryFactory[] support;
	double[] cummulativeProbs;
	boolean uniform;
		
		
	public Itinerary getItinerary(Intersection source, Intersection destination, Car car)
	{
		int index = 0;//default, if the support is 1-long
		if(support.length!=1)
			if(uniform)
				index = state.random.nextInt(support.length);
			else
			{
				double u = state.random.nextDouble();
				//I already know that cummulativeProbs[cummulativeProbs.l-1]==1.
				for(int i=cummulativeProbs.length-2;i>=0;i--)
					if(u>cummulativeProbs[i])
					{
						index = i+1;
						break;
					}//TODO can this be 0????
				//if all these failed, index was already set to 0 
				while(index > 0 && cummulativeProbs[index]==cummulativeProbs[index-1]) index--;//skip the 0 probability items
			}
		return support[index].getItinerary(source, destination, car);
	}

	public void setup(ParamSimState state, Parameter base)
	{
		super.setup(state, base);
		Parameter def = defaultBase();
		int count = state.parameters.getInt(base.push(P_COUNT), def.push(P_COUNT));
		if(count<=0) throw new RuntimeException("A distribution with an empty support it useless.");
		support = new ItineraryFactory[count];
		double[] probs = cummulativeProbs = new double[count];
		for(int i=0;i<count;i++)
		{
			Parameter pi = base.push(i+"");
			Parameter defi = def.push(i+"");
			ItineraryFactory itiF = support[i] = (ItineraryFactory)state.parameters.getInstanceForParameterEq(pi, defi, ItineraryFactory.class);
			itiF.setup(state, pi);
			probs[i] = state.parameters.getDoubleWithDefault(pi.push(P_PROB), defi.push(P_PROB), 1);
			if(probs[i]<=0) throw new RuntimeException("Invalid probability for factory "+i);
		}
		uniform = true;
		double probs0 = probs[0];
		for(int i=1;i<count;i++) 
			if(probs[i]!=probs0)
			{
				uniform = false;
				break;
			}
		if(!uniform)//let us normalize the probs
		{
			double sum = 0;
			for(int i=0;i<count;i++) 
				sum+=probs[i];
			probs[0]/=sum;
			for(int i=1;i<count;i++) 
				probs[i] = probs[i-1]+ probs[i]/sum;
		}
	}

	public Parameter defaultBase() {return new Parameter(P_MUX_ITIN_FACT);}
}
