package sim.util;
import ec.util.*;

/**
 * @author Gabriel Catalin Balan
 * 
 * @see ec.Prototype
 * 
 * alternative to <code>void setup(ParamSimState state, Parameter base);</code>:
 * <code>void setup(int id, Parameter base, ParamSimState state);</code>
 * where <code>id</code> is for efficiency in aggregation scenarios:
 * image that you have SimState has an array of N Parametrizable objects 
 * and they all must know their index (pass it along to others, maybe);
 * this way I avoid theta O(N^2) work.
 */
public interface Parametrizable extends Cloneable
{
	void setup(ParamSimState state, Parameter base);	
	Parameter defaultBase();
	Object simpleClone();
}
