package sim.util;

import sim.engine.*;
import java.util.*;

/**
 *
 * Simple Queue of Steppables, once in, nobody leaves before being served
 * No synchronization is needed because this is used in a mono-thread simulation.
 * 
 * it offers "wake up the next n" functionality
 * 
 * not synchonized, so don't expose it to multi threads. 
 * 
 *	@see http://cgm.cs.mcgill.ca/~msuder/courses/250/lectures/queues/
 *  but improved to hold k elements in an array of length k, not k-1.
 *  
 *   * @author Catalin Gabriel Balan

 */
public class SteppableQueue implements java.io.Serializable
{
	protected Steppable[] array;
	protected int front=0, back=0;
	protected int size = 0;
	public  static final int INITIAL_SIZE = 10;

	public SteppableQueue(int initialSize)
	{
		array = new Steppable[initialSize];
		front = back = size = 0;
	}
	
	public SteppableQueue()
	{
		this(INITIAL_SIZE);
	}
    
    public int size(){return size;}
    public void clear(){front = back = size = 0; }
    
	public Steppable dequeue()
	{
		if (size == 0)
			throw new NoSuchElementException("No More Elements");
		Steppable temp = array[front];
		array[front] = null;
		front = (front + 1) % array.length;
		size--;
		return temp;
	}

	public void enqueue(Steppable e)
	{
		if (size == array.length)
		{
			int len = array.length;
			Steppable[] newArray = new Steppable[2*len];
			if(back > front)
				System.arraycopy(array, front, newArray, 0, size);
			else
			{
				int firstPart = size-front;
				System.arraycopy(array, front, newArray, 0, firstPart);
				System.arraycopy(array, 0, newArray,firstPart, back);
			}
			array = newArray;
			front = 0;//back=size
			array[size] = e;
			back = ++ size;
			return;
		}
		array[back] = e;
		back = (back + 1) % array.length;
		size++;
	}
    
	public String toString()
	{
		String str = "array = |";
		if(size!=0)
			for (int i = 0; i < array.length; i++) 
			{
				if ((front < back && front <= i && i < back) || (back <= front && (front <= i || i < back)))
					str += array[i] + "|";
				else
					str += " |";
			}
		else
			for (int i = 0; i < array.length; i++) 
					str+=" |";
		str += " front = " + front + " back = " + back;
		return str;
	}
	public Iterator iterator()
	{
		return new QueueIteratorF2B(this);
	}
	
	/** Iterator that goes through the elements in the queue front to back (F2B) */
	static class QueueIteratorF2B implements Iterator, java.io.Serializable
	{
		int index = 0;
		SteppableQueue q;
        
		public QueueIteratorF2B(SteppableQueue q) { this.q = q; }
        
		public boolean hasNext()
		{
			return (index < q.size);
		}
			
		public Object next()
		{
			if (index >= q.size) throw new NoSuchElementException("No More Elements");
			int offset = (q.front+index)%q.array.length;
			index++;
			return q.array[offset];
		}
			
		public void remove()
		{
			 throw new UnsupportedOperationException ("call dequeue to remove elements.");
		}
	}
}
