package sim.util.doubledistribution;
import ec.util.*;


/**
 * The Pareto distribution,
 * <br>D(x) = 1-(b/x)^a 
 * <br>P(x) = ab^a/x^(a+1)
 * <br>X = D^-1(U) =b/(1-u)^(1/a) = b*(1-u)^(-1/a)
 * <br>The mean is ab/(a-1).
 * <br>The variance is ab^2/(a-1)^2/(a-2)
 * <br>x>=b !!! //b is min
 * 
 * <p>SIMPLIFIED version (b=1)
 * <br>D(x) = 1-1/x^a 
 * <br>P(x) = a/x^(a+1)
 * <br>X = D^-1(U) =1/(1-u)^(1/a) = (1-u)^(-1/a)
 * <br>the mean is a/(a-1)
 * <br>the variance is a/(a-1)^2/(a-2)
 * <br>x>=1
 *  
 * @author Gabriel Catalin Balan
 */
public class ParetoDoubleDistribution extends DoubleDistribution
{
	double a;
	double b;
	double minusOneOverA;
	double max;
	
	public static final String P_PARETO = "distibution.preto";
	public static final String P_A = "a";
	public static final String P_B = "b";
	public static final String P_MAX = "max";

	public Parameter defaultBase(){	return new Parameter(P_PARETO);}
	
	public void setup(ParameterDatabase parameters, Parameter base, MersenneTwisterFast random)
	{	
		super.setup(parameters, base, random);
		Parameter def = defaultBase();
		a = parameters.getDouble(base.push(P_A), def.push(P_A),0);
		if(a==1) throw new RuntimeException("infinite mean and vaiance");
		if(a==2) throw new RuntimeException("infinite variance");
		if(a==0) throw new RuntimeException("zero variance");
		minusOneOverA =-1d/a;
		b = parameters.getDouble(base.push(P_B), def.push(P_B),0);
		
		max = parameters.getDouble(base.push(P_MAX), def.push(P_MAX),0);
		if(max<0)
			throw new RuntimeException("max value is not specified");
			

	}
	
	public double getDouble()
	{
		
		double v = Math.min(max,b*Math.pow((1.0-rng.nextDouble()),minusOneOverA));
		if(v==0)
		{
			System.out.println("0 in pareto just became 1");
			v = 1;
		}
		if(v<=0)
			System.out.println("negneg");
		return v;
	}	
}