package sim.util.doubledistribution;

import ec.util.*;

/**
 * Uniform distribution: [min, max). 
 * Yes, it cannot reach max.
 *  
 * @author Gabriel Catalin Balan
 *
 */
public class UniformDoubleDistribution extends DoubleDistribution
{
	double min;
	double max;

	public static final String P_UNIF = "distibution.uniform";
	public static final String P_MIN = "min";
	public static final String P_MAX = "max";

	public Parameter defaultBase(){	return new Parameter(P_UNIF);}
	
	public void setup(ParameterDatabase parameters, Parameter base, MersenneTwisterFast random)
	{	
		super.setup(parameters, base, random);
		Parameter def = defaultBase();
		min = parameters.getDouble(base.push(P_MIN), def.push(P_MIN), Double.NEGATIVE_INFINITY);
		if(min<0) throw new RuntimeException("min too small");
		max = parameters.getDouble(base.push(P_MAX), def.push(P_MAX), min);
		if(max<min)
			throw new RuntimeException("max too small");
	}
	
	public double getDouble()
	{
		return (min!=max)? rng.nextDouble()*(max-min)+min : min;
	}
}
