package sim.util.intdistribution;
import ec.util.*;
/**
 *
 * The exponential distribution,<br>
 * 	D(x) = 1-exp(-lambda x) <br>
 *	P(x) = lambda exp(- lambda x)<br>
 * 	X = D^-1(U) =-ln(1- U)/lambda<br>
 *	the mean is equal to the stdev = 1/lambda
 *
 * @author Gabriel Catalin Balan
 */
public class ExponentialIntDistribution extends IntDistribution
{
	int mean, max;
	boolean limited = false;
	public static final String P_EXP = "distibution.exponential";
	public static final String P_MEAN = "mean";
	public static final String P_MAX = "max";
	public static final String P_LIMITED = "limit";

	public Parameter defaultBase(){	return new Parameter(P_EXP);}
	
	public void setup(ParameterDatabase parameters, Parameter base, MersenneTwisterFast random)
	{	
		super.setup(parameters, base, random);
		Parameter def = defaultBase();
		mean = parameters.getInt(base.push(P_MEAN), def.push(P_MEAN));
		if(mean<=0) throw new RuntimeException("mean too small");
		
		limited = parameters.getBoolean(base.push(P_LIMITED), def.push(P_LIMITED), limited);
		if(limited)
		{
			max = parameters.getInt(base.push(P_MAX), def.push(P_MAX));
			if(max< mean) throw new RuntimeException("max too small");
		}
	}
	
	public int getInt()
	{
		int v = (int)Math.round(-Math.log(1.0-rng.nextDouble()) * mean);
		if (! limited)
			return v;
		if(v>=max)
			return max;
		return v;
	}
}