package sim.util.intdistribution;
import ec.util.*;
import sim.util.*;
/**
 * This is the abstract super of a random number generator 
 * producing integers.  Different subclasses implement different 
 * distributions.
 * 
 * 
 * @author Gabriel Balan
 */
public abstract class IntDistribution implements Parametrizable
{
	public static final String P_SEED= "seed";
	public static final String V_TIME= "time";
	public MersenneTwisterFast rng;
	public Parameter defaultBase(){	return null;}
	public Object simpleClone(){return null;}
	
	public void setup(ParamSimState state, Parameter base){setup(state.parameters, base, state.random);}
	public void setup(ParameterDatabase parameters, Parameter base, MersenneTwisterFast random)
	{
		Parameter def = defaultBase();
		long seed;
		String seedImg = parameters.getString(base.push(P_SEED), def.push(P_SEED));
		if(seedImg.equalsIgnoreCase(V_TIME))
			seed = random.nextLong();
			//since i have many RNGs, i cannot seed them all with System.currentTimeMillis()
			//cause they'll have the same seed.
		else
			seed= parameters.getInt(base.push(P_SEED), def.push(P_SEED));
		rng = new MersenneTwisterFast(seed);
	}
	abstract public int getInt();
}
