package sim.util.jfreechart;
import org.jfree.data.statistics.*;
import sim.util.*;
import sim.util.stats.MyTallyInArray;

import java.util.*;
/**
 * @author Gabriel Balan
 *
 */
public abstract class BoxAndWhiskerCalculator
{

	public static BoxAndWhiskerItem calculateBoxAndWhiskerStatistics(double[] data)
	{
		return calculateBoxAndWhiskerStatistics(data, data.length, false);
	}
	
	public static BoxAndWhiskerItem calculateBoxAndWhiskerStatistics(DoubleBag dataBag)
	{
		return calculateBoxAndWhiskerStatistics(dataBag.objs, dataBag.numObjs, false);
	}

	public static BoxAndWhiskerItem calculateBoxAndWhiskerStatistics(DoubleBag dataBag, double outlierCoefficient)
	{
		return calculateBoxAndWhiskerStatistics(dataBag.objs, dataBag.numObjs, false, outlierCoefficient, false);
	}

	public static BoxAndWhiskerItem calculateBoxAndWhiskerStatistics(MyTallyInArray tally)
	{
		tally.sort();//there's a chance data was sorted.
		return calculateBoxAndWhiskerStatistics(tally.getArray().elements(), tally.numberObs(), true);
	}


	public static BoxAndWhiskerItem calculateBoxAndWhiskerStatistics(MyTallyInArray tally, double outlierCoefficient, boolean skipOutliersTriplets)
	{
		tally.sort();//there's a chance data was sorted.
		return calculateBoxAndWhiskerStatistics(tally.getArray().elements(), tally.numberObs(), true, outlierCoefficient, skipOutliersTriplets);
	}
	
	public static double quantile(double[] data, int n, double u)
	{
		u*=(n-1);
		int index1 = (int)u;
		double w1 = u - index1;//frac{u}
		if(w1 ==0) return data[index1];
		int index2 = index1+1;
		double w2 = (w1==0)? 0 : index2 - u;
		return data[index1]*w2 + w1 * data[index2];
	}

	public static BoxAndWhiskerItem calculateBoxAndWhiskerStatistics(double[] data, int n, boolean alreadySorted)
	{
		return calculateBoxAndWhiskerStatistics(data, n, alreadySorted, 1.5, false);
	}
	
	public static BoxAndWhiskerItem calculateBoxAndWhiskerStatistics(double[] data, int n, double outlierCoeficient, boolean skipOutliersTriplets)
	{
		return calculateBoxAndWhiskerStatistics(data, n, false, outlierCoeficient, skipOutliersTriplets);
	}

	public static BoxAndWhiskerItem calculateBoxAndWhiskerStatistics(double[] data, int n, double outlierCoeficient)
	{
		return calculateBoxAndWhiskerStatistics(data, n, false, outlierCoeficient, false);
	}
	
	public static BoxAndWhiskerItem calculateBoxAndWhiskerStatistics(double[] data, int n, boolean alreadySorted, double outlierCoefficient, boolean skipOutliersTriplets)
	{
		if(!alreadySorted)
			java.util.Arrays.sort(data, 0, n);
		
		double sum=0;
		for(int i=0;i<n;i++) sum+=data[i];
		double q1 = quantile(data, n, .25);
		double q2 = quantile(data, n, .5);
		double q3 = quantile(data, n, .75);
		double iqr = q3 - q1;
		double maxWhiskerLen = iqr*outlierCoefficient;
		
		double upperOutlierThreshold = q3 + maxWhiskerLen;
		double lowerOutlierThreshold = q1 - maxWhiskerLen;
//		double upperFaroutThreshold = q3 + (interQuartileRange * 2.0);
//		double lowerFaroutThreshold = q1 - (interQuartileRange * 2.0);
		
		double minRegularValue = data[0], maxRegularValue = data[n-1];
		
		
		List outliers = new ArrayList();
		double lastOutlier = Double.NaN;
		int occurence=1;
		
		for(int i=0; i<n;i++)
		{
			double value = data[i];
			if (value < lowerOutlierThreshold)
			{
				if(skipOutliersTriplets)
					if(value==lastOutlier)
					{
						occurence++;
						if(occurence>2)
							continue;
					}
					else
					{
						occurence=1;
						lastOutlier=value; 
					}
				outliers.add(new Double(value));
				
//				if (value < minOutlier && value >= lowerFaroutThreshold)
//					minOutlier = value;
			}
			else
			{
				minRegularValue = value;
				break;
			}
		}

		lastOutlier = Double.NaN;
		occurence=1;
				
		for(int i=n-1; i>=0;i--)
		{
			double value = data[i];
			
			if (value > upperOutlierThreshold) 
			{
				if(skipOutliersTriplets)
					if(value==lastOutlier)
					{
						occurence++;
						if(occurence>2)
							continue;
					}
					else
					{
						occurence=1;
						lastOutlier=value; 
					}

				outliers.add(new Double(value));
//				if (value > maxOutlier && value <= upperFaroutThreshold) 
//					 maxOutlier = value;
			}
			else
			{
				maxRegularValue = value;
				break;
			}
		}
		
		return new BoxAndWhiskerItem(
								new Double(sum/n),
								new Double(q2),
								new Double(q1),
								new Double(q3),
								new Double(minRegularValue),
								new Double(maxRegularValue),
								new Double(data[0]),
								new Double(data[n-1]),
								outliers);
	}
}
