package sim.util.jfreechart;

import java.util.ArrayList;
import java.util.List;


import org.jfree.data.statistics.*;
import org.jfree.data.xy.*;

import sim.util.Bag;

/**
 * This is an alternative to JFreeChart's DefaultBoxAndWhiskerXYDataset, which
 * only allows a single series and requires pesky Date objects as X values.
 * 
 * 
 * @author Gabriel Balan
 * 
 */
public class BoxAndWhiskerMultiSeriesXYDataset extends AbstractXYDataset
											implements BoxAndWhiskerXYDataset
{
	/**
	 * the "dataset" bag/list constains series of type BoxAndWhiskerXYSeries  
	 * 
	 * Because of removeSeries, I could use a list here, instead of a Bag.
	 * (series don't come and go all that often, so it's not important)
	 */
	//Bag dataSet = new Bag();
	List dataSet = new ArrayList();
	private double outlierCoefficient = 1.5;
	private double faroutCoefficient = 2.0;
	
	
	//I could use two bags (keys and series), but it would waste more space at size-doubling
	public class BoxAndWhiskerXYSeries
	{
		/** the "seriesData" contains XvalueAndBoxAndWhiskerItem elements */
		//I could use two bags, but it would waste more space at size-doubling
		Bag seriesData;
		Comparable key;
		public BoxAndWhiskerXYSeries(Comparable key)
		{
			seriesData = new Bag();
			this.key = key;
		}
		public void clear(){seriesData.clear();}
		public void add(Number xValue, BoxAndWhiskerItem bwItem)
		{
			seriesData.add(new XvalueAndBoxAndWhiskerItem(xValue, bwItem));
		}
	}
	
	public class XvalueAndBoxAndWhiskerItem
	{
		BoxAndWhiskerItem bwItem;
		Number xValue;
		public XvalueAndBoxAndWhiskerItem(Number xValue, BoxAndWhiskerItem bwItem)
		{
			this.bwItem = bwItem;
			this.xValue = xValue;
		}
		//If I'll remove duplicates myslef, I should add fields to set these things
	}

//	public BoxAndWhiskerMultiSeriesXYDataset()
//	{
////		System.out.println("Would my many series share a renederer?");
////		System.out.println("A B&W renderer holds the color. I need each series has it's own.");
//		//other renderers get the color from 
//		//getSeriesPaint, getItemPaint (from AbstractRenederer)
//		//SO WE SHOULD BE GOOD TO GO.
//	}

	public void addSeries(String seriesKey)
	{
		addSeries(new BoxAndWhiskerXYSeries(seriesKey));
	}

	public void addSeries(BoxAndWhiskerXYSeries series)
	{
		dataSet.add(series);
	}

	public void addDataPoint(int seriesIndex, Number xValue, BoxAndWhiskerItem bwItem)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		seriesData.add(new XvalueAndBoxAndWhiskerItem(xValue, bwItem));
	}

	public void removeSeries(int seriesIndex)
	{
		dataSet.remove(seriesIndex);
	}


	/** Returns the median-value for the specified series and item. */
	public Number getMeanValue(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getMean();
	}

	public Number getMedianValue(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getMedian();
	}

	public Number getQ1Value(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getQ1();
	}

	public Number getQ3Value(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getQ3();
	}

	public Number getMinRegularValue(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getMinRegularValue();
	}

	public Number getMaxRegularValue(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getMaxRegularValue();
	}

	public Number getMinOutlier(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getMinOutlier();
	}

	public Number getMaxOutlier(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getMaxOutlier();
	}

	public List getOutliers(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getOutliers();
	}

	public double getOutlierCoefficient()
	{
		return outlierCoefficient;
	}

	public double getFaroutCoefficient()
	{
		return faroutCoefficient;
	}


	public int getItemCount(int seriesIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		return seriesData.numObjs;
	}

	public Number getX(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		return ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).xValue;
	}

	public Number getY(int seriesIndex, int itemIndex)
	{
		Bag seriesData = ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).seriesData;
		BoxAndWhiskerItem item  = ((XvalueAndBoxAndWhiskerItem)seriesData.objs[itemIndex]).bwItem;
		return item.getMean();
	}


	public int getSeriesCount()
	{
		return dataSet.size();
	}

	public Comparable getSeriesKey(int seriesIndex)
	{
		return ((BoxAndWhiskerXYSeries)dataSet.get(seriesIndex)).key;
	}	
}
