package sim.util.stats;
import java.io.*;

import cern.colt.list.DoubleArrayList;
import sim.util.*;
import ec.util.*;
import umontreal.iro.lecuyer.util.PrintfFormat;
/**
 * @author Gabriel Balan
 *
 */
public /*strictfp*/ class RankTTestAnalysis 
{
	String[] treatmentNames;
	Record[][] treatments;
	static class Record
	{
		int treatmentIndex;//index of owner
		float rank;
		double value;
		public String toString(){return value+"\t"+rank+"\t"+treatmentIndex;}
	}
	
	static SortComparator comparator = new SortComparator(){
		public boolean lt(Object a, Object b){ return ((Record)a).value>((Record)b).value;}
		public boolean gt(Object a, Object b){ return ((Record)a).value<((Record)b).value;}
	};
		
	
	public static final void main(String[] args)
	{
		RankTTestAnalysis rta;
		try
		{
			switch(args.length)
			{
				case 0: System.err.println("no args!");System.exit(-1);
				case 1:
						//all treatments are in the same file, [names on the first line]
						rta = new RankTTestAnalysis(new BufferedReader(new FileReader(args[0])));
						break;
				default:
						//each treatment has its own file, use the file name as treatment name
						rta = new RankTTestAnalysis(args.length);
						for(int i=args.length-1;i>=0;i--)
						{
							String fname = args[i];
							String treatmentName = new File(fname).getName();
							rta.addTreatment(treatmentName, new BufferedReader(new FileReader(fname)));
						}
			}
			System.out.println(rta.getValues(0).formatConfidenceIntervalStudent(0.05));
			System.out.println(rta.getValues(1).formatConfidenceIntervalStudent(0.05));
			
			rta.computeRanks(0, 1);
			rta.printTreatment(0);
			rta.printTreatment(1);
			
			umontreal.iro.lecuyer.stat.Tally a, b;
			System.out.println((a = rta.getRanks(0)).formatConfidenceIntervalStudent(0.05));
			System.out.println((b = rta.getRanks(1)).formatConfidenceIntervalStudent(0.05));

			
			StatisticUtilities.tTest(a, b, 0.05, true);
			
			//TODO for n treatments do all n^2/2 comparissons 
			//TODO use Bonferroni corretion.
			//Bonferroni decreases alpha (prob of type I error: saying there's a difference
			//when in fact there's not), but increases beta (prob of type II error, the
			//chance of missing a difference).
		}
		catch(FileNotFoundException ex)	{ex.printStackTrace();}
		catch(IOException ex)			{ex.printStackTrace();}

	}
	
	public RankTTestAnalysis(BufferedReader br) throws IOException
	{	
		String separators = "\t";
		treatmentNames = br.readLine().split(separators);
		int n = treatmentNames.length;
		treatments = new Record[n][];
		Bag[] bags = new Bag[n];
		for(int i=0;i<n;i++) bags[i] = new Bag(100);
		String line;
		while((line=br.readLine())!=null)
		{
			String[] numberImages = line.split(separators); 
			for(int i=0;i<numberImages.length;i++)
			{
				String image = numberImages[i];
				if(image.equals("")) continue;
				Record r = new Record();
				r.treatmentIndex = i;
				r.value = Double.parseDouble(image);
				bags[i].add(r);
			}
		}
		for(int i=0;i<n;i++)
		{
			Bag b = bags[i];
			Record[] data = treatments[i] =new Record[b.numObjs];
			System.arraycopy(b.objs, 0, data, 0, b.numObjs);
			QuickSort.qsort(data, comparator);
		}
		
	}

	public RankTTestAnalysis(StatAnalysis sa)
	{
		this(sa.treatments.length);
		for(int i=treatments.length-1;i>=0;i--)
			addTreatment(sa.treatments[i]);
	}

	private int unsetTreatments=0;
	public RankTTestAnalysis(int nTreatments)
	{	
		unsetTreatments = nTreatments;
		treatmentNames = new String[nTreatments];
		treatments = new Record[nTreatments][];
	}

	Bag tmp = new Bag(100);
	private void addTreatment(String name, BufferedReader br)
	{
		unsetTreatments--;
		treatmentNames[unsetTreatments]=name;
		tmp.clear();
		String line;
		try{
			while((line=br.readLine())!=null&& !line.equals(""))
			{
				Record r = new Record();
				r.value  = Double.parseDouble(line);
				r.treatmentIndex = unsetTreatments;
				tmp.add(r);
			}
		}catch(IOException ex){ex.printStackTrace();}		
		Record[] data = treatments[unsetTreatments] = new Record[tmp.numObjs];
		System.arraycopy(tmp.objs, 0, data, 0, data.length);
		QuickSort.qsort(data, comparator);
	}
	
	private void addTreatment(TallyInABag sample)
	{
		unsetTreatments--;
		treatmentNames[unsetTreatments]=sample.getName();
		int size = sample.data.numObjs;
		Record[] data = treatments[unsetTreatments] = new Record[size];		
		if(!sample.sorted) sample.sort();
		//tally-in-a-bag sorts the numbers in increasing order.
		//but for legacy reasons (i.e. Wineberg & Christensen did so in their GECCO03 tutorial)
		//the ranks are computed for decreasing order. So... I load the data in reverse.
		for(int i= size-1;i>=0;i--)
		{
			Record r = data[i] = new Record();
			r.treatmentIndex = unsetTreatments;
			r.value = sample.data.objs[size-1-i];
		}
	}
	
	private void addTreatment(MyTallyInArray sample)
	{
		unsetTreatments--;
		treatmentNames[unsetTreatments]=sample.getName();
		int size = sample.numberObs();
		Record[] data = treatments[unsetTreatments] = new Record[size];
		DoubleArrayList sampleData = sample.getArray();
		if(!sample.sorted) sampleData.sort();
		//MyTallyInArray sorts the numbers in increasing order.
		//but for legacy reasons (i.e. Wineberg & Christensen did so in their GECCO03 tutorial)
		//the ranks are computed for decreasing order. So... I load the data in reverse.

		double[] sampleArray = sampleData.elements();
		for(int i= size-1;i>=0;i--)
		{
			Record r = data[i] = new Record();
			r.treatmentIndex = unsetTreatments;
			r.value = sampleArray[size-1-i];
		}
	}

	static private double[] tmpInterval = new double[2];

	private MyTallyInArray getRanks(int i)
	{
		Record[] treatment = treatments[i];
		int n = treatment.length;
		MyTallyInArray s = new MyTallyInArray(n)
		{
			//replace "confidence interval for mean" with "confidence interval for median"
			public String formatConfidenceIntervalStudent(double d, int i)
			{
				PrintfFormat printfformat = new PrintfFormat();
				confidenceIntervalStudent(d, tmpInterval);
				double mean = tmpInterval[0];
				double conf = tmpInterval[1];
				printfformat.append("  " + 100D * d + "%");
				printfformat.append(" confidence interval on ranks: (");
				printfformat.append(7 + i, i, i - 1, mean - conf).append(',');
				printfformat.append(7 + i, i, i - 1, mean + conf).append(" )\n");
				return printfformat.toString();
			}
		};
		s.setName(treatmentNames[i]);
		for(int j=0;j<n;j++) s.add(treatment[j].rank);
		return s;
	}

	private MyTallyInArray getValues(int i)
	{
		Record[] treatment = treatments[i];
		int n = treatment.length;
		MyTallyInArray s = new MyTallyInArray(n);
		s.setName(treatmentNames[i]);
		for(int j=0;j<n;j++) s.add(treatment[j].value);
		return s;
	}
	
	private void computeRanks(int a, int b){computeRanks(treatments[a], treatments[b]);}

	private void computeRanks(Record[] treatment_a, Record[] treatment_b)
	{
		//I assume the treatments are already sorted,
		//so merge-sort can be replaced by merely merging.
		int sizeA = treatment_a.length;
		int sizeB = treatment_b.length;
		int sizeAB = sizeA+sizeB;
		Record[] mergedData = new Record[sizeAB];
		Record rA, rB;
		int iA=0, iB=0,iAB=0;
		while(iA<sizeA && iB<sizeB)
			if((rA = treatment_a[iA]).value > (rB = treatment_b[iB]).value)
				{mergedData[iAB++] = rA; iA++;}
			else
				{mergedData[iAB++] = rB; iB++;}
		
		if(iA<sizeA) System.arraycopy(treatment_a, iA, mergedData, iAB, sizeA-iA);
		if(iB<sizeB) System.arraycopy(treatment_b, iB, mergedData, iAB, sizeB-iB);
		
		//compute the indices
		int tieStart = 0;
		Record lastR = mergedData[0];
		int tieRankSum = 0;
		lastR.rank= 0;
		
		for(int i=1; i<sizeAB;i++)
		{
			Record thisR = mergedData[i];
			thisR.rank = i;
			if(thisR.value != lastR.value)
			{
				if(i-tieStart>1)
				{
					float rank = ((float)tieRankSum)/(i-tieStart);
					for(int j=tieStart;j<i;j++) mergedData[j].rank = rank;
				}
				tieStart = i;
				tieRankSum = i;
			}
			else
				tieRankSum += i;
			lastR = thisR;
		}
		//in the tutorial the indices start from 1, and in here they're 0 based, so here's the correction:
		for(int i=0; i<sizeAB;i++) mergedData[i].rank++;
	}
	
	private void printTreatment(int i)
	{
		System.out.println("Treatment "+treatmentNames[i]);
		Record[] data = treatments[i];
		for(int j =0;j<data.length;j++)
		{
			Record r = data[j];
			System.out.println(r.value+"\t"+r.rank);
		}
	}
	
	/**
	 * 
	 * 
	 * @param alpha confidence level
	 * 
	 * 
	 * Bonferroni decreases alpha (prob of type I error: saying there's a difference
	 * when in fact there's not), but increases beta (prob of type II error, the
	 * chance of missing a difference).
	 */
	public void printPartialOrderRelationMatrix(double alpha, StatAnalysis.MultiComparisonCorrection correction)
	{
		int n = treatments.length;
		printRankedTTestRelationMatrix(alpha, n*(n-1)/2, correction);
	}	
	
	public void printRankedTTestRelationMatrix(double alpha, int nTests, StatAnalysis.MultiComparisonCorrection correction)	
	{
		int n = treatments.length;
		double individualAlpha = correction.getIndividualAlpha(alpha,nTests);
		//TODO if I only care about who's the best, do I need only n comparisons?
		//(I should specify if higher is better or lower is). 

		System.out.println();
		System.out.println("RankedTtest\nalpha= "+alpha+"\t Bonferroni_alpha= "+individualAlpha);

		int[][] relationMatrix = new int[n][n]; 
		//+1 >
		//-1 <
		// 0 = 
		for(int i=0;i<n;i++)
		{
			Record[] ti = treatments[i];
			for(int j=0;j<n;j++)
			{
				if(i==j) continue;
				Record[] tj = treatments[j];
				computeRanks(ti,tj);
				
				MyTallyInArray ranks_i = getRanks(i);
				MyTallyInArray ranks_j = getRanks(j);
				
				boolean result = StatisticUtilities.tTest(ranks_i, ranks_j, individualAlpha, false, false);

				if(result)
					if(ranks_i.average()<ranks_j.average())
				{	//I know you'd expect it the other way around, but the tallies are sorted in descending order, so more is less
						relationMatrix[i][j]=+1;
						relationMatrix[j][i]=-1;
					}
					else
					{
						relationMatrix[i][j]=-1;
						relationMatrix[j][i]=+1;
					}
			}
		}
		StatAnalysis.printRelationMatrix(relationMatrix);
	}	
	
	public void printTreatmentNames()
	{
		for(int i=0;i<treatmentNames.length;i++)
		{
			System.out.println(i+"\t"+treatmentNames[i]);
		}
	}

	public void printEquivalenceClassesCIs(double alpha, StatAnalysis.MultiComparisonCorrection correction)	
	{	
		int n = treatments.length;
		printRankCIRelationMatrix(alpha, n*(n-1)/2, correction);
	}
		
	public void printRankCIRelationMatrix(double alpha, int ntests, StatAnalysis.MultiComparisonCorrection correction)
	{
		int n = treatments.length;
		double individualAlpha = correction.getIndividualAlpha(alpha,ntests);

		System.out.println();
		System.out.println("RankedTtest-CIS\nalpha= "+alpha+"\t Bonferroni_alpha= "+individualAlpha);

		int[][] relationMatrix = new int[n][n]; 
		double[] CIi = new double[2];
		double[] CIj = new double[2];
		//+1 >
		//-1 <
		// 0 = 
		for(int i=0;i<n;i++)
		{
			Record[] ti = treatments[i];
			for(int j=0;j<n;j++)
			{
				if(i==j) continue;
				Record[] tj = treatments[j];
				computeRanks(ti,tj);
				
				MyTallyInArray ranks_i = getRanks(i);
				MyTallyInArray ranks_j = getRanks(j);

				ranks_i.confidenceIntervalStudent(1-individualAlpha, CIi);
				ranks_j.confidenceIntervalStudent(1-individualAlpha, CIj);
				
				double low_j = CIj[0]-CIj[1];
				double high_j = CIj[0]+CIj[1];
				double low_i = CIi[0]-CIi[1];
				double high_i = CIi[0]+CIi[1];
				

				if(low_j>high_i)
				{	//I know you'd expect it the other way around, but the tallies are sorted in descending order, so more is less
					relationMatrix[i][j]= 1;
					relationMatrix[j][i]=-1;
				}
				else
				if(low_i>high_j)
				{
					relationMatrix[i][j]=-1;
					relationMatrix[j][i]=+1;
				}
			}
		}
		StatAnalysis.printRelationMatrix(relationMatrix);
	}	

}
