package sim.util.stats;
import java.io.*;

import ec.util.*;

/**
 * @author Gabriel Balan
 *
 */
public /*strictfp*/ class StatAnalysis 
{
	MyTallyInArray[] treatments;
	
	
	public static abstract class ConfidenceIntervalFactory
	{
		public boolean sortByCICenters;
		public ConfidenceIntervalFactory(boolean sort){sortByCICenters = sort;}
		public abstract void getConfidenceInterval(MyTallyInArray tally, double alpha, double[] centerAndRadius);
	};

	public static class MedianConfidenceIntervalFactory extends ConfidenceIntervalFactory
	{
		public MedianConfidenceIntervalFactory(boolean sortByCenters){super(sortByCenters);}
		public void getConfidenceInterval(MyTallyInArray tally, double alpha, double[] centerAndRadius)
		{
			tally.confidenceIntervalMedian(alpha, centerAndRadius);
		}
		public String toString(){return "Confidence Intervals for the MEDIAN";}
	}
	
	public static class AverageConfidenceIntervalFactory extends ConfidenceIntervalFactory
	{
		public AverageConfidenceIntervalFactory(boolean sortByCenters){super(sortByCenters);}
		public void getConfidenceInterval(MyTallyInArray tally, double alpha, double[] centerAndRadius)
		{
			tally.confidenceIntervalStudent(1-alpha, centerAndRadius);
			///same thing as: 
//			tally.confidenceIntervalMean(alpha, centerAndRadius);
			
			/// Tally.confidenceIntervalStudent uses the confidence coefficient!
			/// TallyInABag.confidenceIntervalMean uses the significance level!
			///
			///(somehow the first has 1 more decimal than the second.)
		}
		public String toString(){return "Confidence Intervals for the AVERAGE";}
	}
	
	static final ConfidenceIntervalFactory AVERAGE_CI_FACTORY = new AverageConfidenceIntervalFactory(false);
	static final ConfidenceIntervalFactory MEDIAN_CI_FACTORY = new MedianConfidenceIntervalFactory(false);	
	static final ConfidenceIntervalFactory AVERAGE_CI_FACTORY_wSORT = new AverageConfidenceIntervalFactory(true);
	static final ConfidenceIntervalFactory MEDIAN_CI_FACTORY_wSORT = new MedianConfidenceIntervalFactory(true);
	
	
		
	public static final void main(String[] args)
	{
		StatAnalysis sa;
		try
		{
			switch(args.length)
			{
				case 0: System.err.println("no args!");System.exit(-1);
				case 1:
						//all treatments are in the same file, [names on the first line]
						sa = new StatAnalysis(new BufferedReader(new FileReader(args[0])));
						break;
				default:
						//each treatment has its own file, use the file name as treatment name
						sa = new StatAnalysis(args.length);
						for(int i=args.length-1;i>=0;i--)
						{
							String fname = args[i];
							String treatmentName = new File(fname).getName();
							sa.addTreatment(treatmentName, new BufferedReader(new FileReader(fname)));
						}
			}
			double alpha = Double.parseDouble(System.getProperty("alpha", "0.05"));
			int ntreatments = Integer.parseInt(System.getProperty("number.treatments", ""+sa.treatments.length));
			int ncomparisons = Integer.parseInt(System.getProperty("number.comparisons", ""+(ntreatments*(ntreatments-1)/2)));
			boolean verbosity = Boolean.getBoolean("verbosity");
			sa.pseudoANOVAEquivalenceClasses(alpha, true, BONFERRONI_CORRECTION, ncomparisons, verbosity);

//			//			sa.analyze(alpha, BONFERRONI_CORRECTION);
//			//			sa.analyzeNPDelayNodes(alpha, BONFERRONI_CORRECTION);
//						sa.pseudoANOVAEquivalenceClasses(alpha, true, BONFERRONI_CORRECTION);
//						//TODO sidak is tighter (i.e. better) 
//						//1. but are there additional assumptions? NO, THEY BOTH REQUIRE INDEPENDENCE.
//						//2. is it worse the trouble of having to explain it?
//			//			sa.pseudoANOVAEquivalenceClasses(alpha, false, SIDAK_CORRECTION);

		}
		catch(FileNotFoundException ex)	{ex.printStackTrace();}
		catch(IOException ex)			{ex.printStackTrace();}

	}
	
	public void analyze(double alpha, MultiComparisonCorrection correction)
	{
		sortByName();
		RankTTestAnalysis rtta = new RankTTestAnalysis(this);
		printRelationMatrix(alpha, AVERAGE_CI_FACTORY, 
//									null,
//									new NameTallyDescriptor(),
//									new NameAndNormalityTallyDescriptor(),
									new AnovaAssumptionsTallyDescriptor(),
									correction);
		printRelationMatrix(alpha, MEDIAN_CI_FACTORY, null, correction);
		rtta.printEquivalenceClassesCIs(alpha, correction);
		rtta.printPartialOrderRelationMatrix(alpha, correction);
		rtta.printTreatmentNames();
	}
	
	

	/**
	 * 4 mixed treatments (no delay, seq delay, para delay, 2 delays)=> 4(4-1)/2=6 tests 
	 * n-4 treatments => (n-4)/2 tests 
	 * 
	 * (n-4+12)/2 = (n-8)/2
	 */
	public void analyzeNPDelayNodes(double alpha, MultiComparisonCorrection correction)
	{
		int n = treatments.length;
		int ntests = (n-4)/2 + 4*(4-1)/2;//for E1, E2, maze10
//		int ntests = (n-7)/2 + 4*(4-1)/2 +3*(3-1)/2;//maze7
		RankTTestAnalysis rtta = new RankTTestAnalysis(this);
		printRelationMatrix(alpha,
								ntests,
								AVERAGE_CI_FACTORY, 
								null,
//								new NameTallyDescriptor(),
//								new NameAndNormalityTallyDescriptor(),
								BONFERRONI_CORRECTION);
		printRelationMatrix(alpha, ntests, MEDIAN_CI_FACTORY, null, BONFERRONI_CORRECTION);
		rtta.printRankCIRelationMatrix(alpha, ntests, correction);
		rtta.printRankedTTestRelationMatrix(alpha, ntests, correction);
		rtta.printTreatmentNames();
	}
	
	public static final SortComparator SORT_BY_AVERAGE_COMPARATOR = new SortComparator(){
			public boolean lt(Object a, Object b){return ((MyTallyInArray)a).average()<((MyTallyInArray)b).average();}
			public boolean gt(Object a, Object b){return ((MyTallyInArray)a).average()>((MyTallyInArray)b).average();}};

	public static final SortComparator SORT_BY_NAME_COMPARATOR = new SortComparator(){
		public boolean lt(Object a, Object b){return ((MyTallyInArray)a).getName().compareToIgnoreCase(((MyTallyInArray)b).getName())<0;}
		public boolean gt(Object a, Object b){return ((MyTallyInArray)a).getName().compareToIgnoreCase(((MyTallyInArray)b).getName())>0;}};

	private void sortByName()
	{
		QuickSort.qsort(treatments, SORT_BY_NAME_COMPARATOR);
	}
	
	
	public static interface TallyDescriptor
	{
		public String describe(MyTallyInArray t);
		public String tableHead();		
	}
	public static class NameTallyDescriptor implements TallyDescriptor
	{
		public String describe(MyTallyInArray t){return t.getName();}
		public String tableHead(){return "Name";}
	}
	private static MyTallyInArray THEORETICAL_NORMAL_DIST =  null;
	public static class NameAndNormalityTallyDescriptor extends NameTallyDescriptor
	{
		public String describe(MyTallyInArray t)
		{
			int n = t.numberObs();
			if(THEORETICAL_NORMAL_DIST==null || THEORETICAL_NORMAL_DIST.numberObs()!=n)
				THEORETICAL_NORMAL_DIST = StatisticUtilities.getNormalDistQuantile(n);
			double r2 = StatisticUtilities.linearRegression(t, THEORETICAL_NORMAL_DIST, null);
			return super.describe(t)+"\t"+r2;
		}
		public String tableHead(){return super.tableHead()+"\tnormality_R^2";}
	}

	/* name, normality score and stdev */
	private static class AnovaAssumptionsTallyDescriptor extends  NameAndNormalityTallyDescriptor
	{
		public String describe(MyTallyInArray t)
		{
			return super.describe(t)+"\t"+t.standardDeviation();
		}
		public String tableHead(){return super.tableHead()+"\tstdev";}		
	}
	static final TallyDescriptor ANOVA_ASSUMPTIONS_DESCRIPTOR = new AnovaAssumptionsTallyDescriptor();

	
	public void printRelationMatrix(double alpha, ConfidenceIntervalFactory ciFactory, TallyDescriptor descriptionFactory, MultiComparisonCorrection correction)	
	{
		int n = treatments.length;
		printRelationMatrix(alpha, n*(n-1)/2, ciFactory, descriptionFactory, correction);
	}
	/**
	 * 
	 * @param alpha confidence level
	 * 
	 * Bonferroni decreases alpha (prob of type I error: saying there's a difference
	 * when in fact there's not), but increases beta (prob of type II error, the
	 * chance of missing a difference).
	 */
	public void printRelationMatrix(double alpha, int ntests, 
									ConfidenceIntervalFactory ciFactory,
									TallyDescriptor descriptionFactory, 
									MultiComparisonCorrection correction)
	{
		double individualAlpha = correction.getIndividualAlpha(alpha, ntests);
		
		System.out.println();
		System.out.println(ciFactory+"\nalpha=\t"+alpha+"\t Bonferroni_alpha=\t"+individualAlpha);

		int n = treatments.length;
		/*
		* For each treatment I keep a double[3]: the first two positions are filled by the 
		* confidence interval factory (center and radius), the last position holds the index 
		* of the treatment. I need this cause I want to sort the confidence intervals by the
		* centers. Plus, the CI factory does not mind longer double[]s.
		*/
		double[][] cis = new double[n][3];
		
		for(int i=0;i<n;i++)
		{
			double[] ci = cis[i];
			ci[2]=i;
			ciFactory.getConfidenceInterval(treatments[i], individualAlpha, ci);
		}

		if(ciFactory.sortByCICenters)	//sort the confidence intervals by the center
			QuickSort.qsort(cis, new SortComparator(){
				public boolean lt(Object a, Object b){return ((double[])a)[0]<((double[])b)[0];}
				public boolean gt(Object a, Object b){return ((double[])a)[0]>((double[])b)[0];}
			});

		if(descriptionFactory!=null)
		{
			System.out.println("\t"+descriptionFactory.tableHead()+"\tLow_bound\tCenter\tUpper_bound\t");
			for(int i=0;i<n;i++)
			{
				double[] ci = cis[i];
				System.out.println(i+"\t"+descriptionFactory.describe(treatments[(int)ci[2]])+"\t"+
								 (ci[0]-ci[1])+"\t"+ci[0] +"\t"+(ci[0]+ci[1]));
			}
			System.out.println();
		}
				
		int[][] relationMatrix = new int[n][n]; 
		//+1 >
		//-1 <
		// 0 = 
		for(int i=0;i<n;i++)
		{
			double low_i = cis[i][0]-cis[i][1];
			double high_i = cis[i][0]+cis[i][1];
			for(int j=0;j<n;j++)
			{
				if(i==j) continue;
				double low_j  = cis[j][0]-cis[j][1];
				double high_j = cis[j][0]+cis[j][1];

				if(low_j>high_i)
				{
					relationMatrix[i][j]=-1;
					relationMatrix[j][i]= 1;
				}
				else
				if(low_i>high_j)
				{
					relationMatrix[i][j]= 1;
					relationMatrix[j][i]=-1;
				}
			}
		}
		printRelationMatrix(relationMatrix);
	}
	
	
	/**
	 * this assumes that the relation matrix is a parital order relation already sorted.
	 */	
	static public void printEquivalenceClasses(int[][] relationMatrix)
	{
		int n = relationMatrix.length;
		for(int i=0;i<n;i++)
		{
			int[] row_i=relationMatrix[i];
			int last = row_i[0];
			int current;
			for(int j=1;j<n;j++)
			{
				current = row_i[j];
				if(current>last)
				{
					System.out.println("Not a partial order relation");
					printRelationMatrix(relationMatrix);
					return;
				}
				last = current;
			}
		}

		//compute the firt treatment's class
		int lastClassStart = 0;
		int lastClassEnd = 0;
		int currentTreatment = 0;
		int[] currentRow = relationMatrix[0];
		while(lastClassEnd<(n-1) && currentRow[lastClassEnd+1]==0) //0 is =
			lastClassEnd++;
		if(lastClassEnd != 0)
			System.out.println(0+"\t->\t"+lastClassEnd);
		//if I print the lines instead of the classes 
		//I should not print anything
		else
			System.out.println(0);
		
		if(lastClassEnd==n-1) return;

		//compute the classes for the rest of the treatments
		for(int t=1;t<n;t++)
		{
			currentRow = relationMatrix[t];
			int start = lastClassStart;
			while(currentRow[start]>0) start++;
			int last = lastClassEnd;
			while(last<(n-1) && currentRow[last+1]>=0) last++;
			
			if(last ==lastClassEnd) continue;
			lastClassEnd = last; lastClassStart = start;
			if(lastClassEnd != t)
				System.out.println(t+"\t->\t"+lastClassEnd);
			else
			//if I print the lines instead of the classes 
			//I should not print anything
				System.out.println(t);
			if(lastClassEnd==n-1) return;
		}
	}
	
	/**
	 * @param relationMatrix = statistical difference between treatments.
	 * M[i][j] < 0 means
	 */
	static public void printRelationMatrix(int[][] relationMatrix)
	{
		int n = relationMatrix.length;
		System.out.print("_\t|");
		for(int j=0;j<n;j++)
			System.out.print(j+"\t");
		System.out.println();
		for(int i=0;i<n;i++)
		{
			System.out.print(i+"\t|");
			for(int j=0;j<n;j++)
			{
				int rel = relationMatrix[i][j];
				String s = (rel<0)? "<" :((rel>0)? ">" :"="); 
				System.out.print(s+"\t");
			}
			System.out.println("|");
		}
	}


	static interface MultiComparisonCorrection{double getIndividualAlpha(double alpha, int comparisons);}
	/**
	 * The Bonferroni correction assumes that the tests are independent.
	 * 
	 * This is an approximation based on the approximation (1 - alpha)^k = 1 - k alpha, 
	 * possible since alpha is going to be very small.
	 * @see http://www.sghms.ac.uk/depts/phs/guide/methods.htm#bonf
	 */
	static class BonferroniCorrection implements MultiComparisonCorrection
	{
		public String toString(){return "Bonferroni";}
		public double getIndividualAlpha(double alpha, int comparisons){return alpha/comparisons;}
	}
	public static final BonferroniCorrection BONFERRONI_CORRECTION = new BonferroniCorrection();
	/**
	 * This uses the exact formula alpha = 1-(1- indiv_alpha)^k =>
	 * indiv_alpha = (1-alpha)^1/k.
	 * 
	 * One can use it if the contrasts are planned in advance of the experiment (a priori) 
	 * and done in place of the analysis of variance the relationship.[...] When comparisons are 
	 * performed after the data have been examined (a posteriori) or subjected to an analysis of
	 * variance then controlling the experiment - wise error rate requires an even larger penalty.
	 *
	 * 
	 * Note that the value of ae estimated under the Dunn-Sidak correction assumes that all contrasts [comparisons]
	 * performed are mutually independent. [But then again, so does Bonferroni. I see no problem with Sidak]
	 * 
	 * 
	 * @see http://userwww.sfsu.edu/~efc/classes/biol458/multcomp/multcomp.htm
	 */
	static class SidakCorrection implements MultiComparisonCorrection
	{
		public String toString(){return "Sidak";}
		public double getIndividualAlpha(double alpha, int comparisons){return 1d- Math.pow(1-alpha,1d/comparisons);}
	}
	public static final SidakCorrection SIDAK_CORRECTION = new SidakCorrection();



	public void pseudoANOVAEquivalenceClasses(double alpha, boolean useDescriptor, MultiComparisonCorrection correction, boolean verbosity)
	{
		printEquivalenceClasses(repeatedTestsRelationMatrix(
									alpha, 
									useDescriptor, 
									correction, 
									SORT_BY_AVERAGE_COMPARATOR, verbosity));
	}

	/**
	 * this goes for the mean only
	 * It's called pseudoANOVA cause it likes normality and homegeneous variances to transform the relation matrix
	 * into equivalence classes.
	 * 
	 * I does repeated T-tests and no ANOVA, since it might contradict any post hoc test except for the Scheffe's.
	 * My oppinion to this date is that ANOVA is for lazy people, and I don't perform the O(n^2) T-tests by hand, 
	 * so I don't mind the extra effort.
	 */
	public void pseudoANOVAEquivalenceClasses(double alpha, boolean useDescriptor, MultiComparisonCorrection correction, int comparissons, boolean verbosity)
	{
		printEquivalenceClasses(repeatedTestsRelationMatrix(
									alpha, 
									useDescriptor, 
									correction, 
									comparissons, 
									SORT_BY_AVERAGE_COMPARATOR, verbosity));
	}

	public int[][] repeatedTestsRelationMatrix(double alpha, 
												boolean useDescriptor, 
												MultiComparisonCorrection correction, 
												SortComparator sortComparator,//can be null 
												boolean verbosity)
	{
		int n = treatments.length;
		return repeatedTestsRelationMatrix(alpha, useDescriptor, correction, n*(n-1)/2, sortComparator, verbosity);
	}
	
	public int[][] repeatedTestsRelationMatrix(double alpha, 
												boolean useDescriptor, 
												MultiComparisonCorrection correction, 
												int comparissons, 
												SortComparator sortComparator, //can be null
												boolean verbosity)
	{
		double individualAlpha = correction.getIndividualAlpha(alpha, comparissons);	
		
		int n = treatments.length;
		
		if(sortComparator!= null) QuickSort.inssort(treatments, sortComparator);
		//I use insertion sort cause the number of treatments is so small it does not matter what I use
		//and the quicksort (although deterministic now) might turn into randomized 
		//and then MooStatAnalysis would be screwed, cause it depends on
		//the analyses for the objectives ordering the treatments the same way

		if(useDescriptor)
		{
			System.out.println("Repeated T-tests\nalpha\t"+alpha+"\t"+correction+"_alpha=\t"+individualAlpha);

			System.out.println("\t"+ANOVA_ASSUMPTIONS_DESCRIPTOR.tableHead()+"\tAverage");
			for(int i=0;i<n;i++)
			{
				MyTallyInArray treatment = treatments[i];
				System.out.println(i+"\t"+ANOVA_ASSUMPTIONS_DESCRIPTOR.describe(treatment)+"\t"+ treatment.average());
			}
			System.out.println();
		}		
		
		int[][] relationMatrix = new int[n][n]; 
		//+1 >
		//-1 <
		// 0 = 
		for(int i=0;i<n;i++)
		{
			MyTallyInArray ti = treatments[i];
			for(int j=i+1;j<n;j++)
			{
				MyTallyInArray tj = treatments[j];
				
				boolean result = StatisticUtilities.tTest(ti, tj, individualAlpha, false, verbosity);

				if(result)
					if(ti.average()>tj.average())
					{
						relationMatrix[i][j]=+1;
						relationMatrix[j][i]=-1;
					}
					else
					{
						relationMatrix[i][j]=-1;
						relationMatrix[j][i]=+1;
					}
			}
		}
		return(relationMatrix);
	}

	
//	private void swap(int[][] matrix, int i, int j)
//	{
//		int[] tmpLine = matrix[i];
//		matrix[i]=matrix[j];
//		matrix[j]=tmpLine;
//		int n = matrix.length;
//		int tmp;
//		for(int k=0;k<n;k++)
//		{
//			int[] line_k = matrix[k];
//			tmp = line_k[i];
//			line_k[i]=line_k[j];
//			line_k[j]=tmp;
//		}
//	}
	
	public StatAnalysis(BufferedReader br) throws IOException
	{	
		String separators = "\t";
		
		String[] treatmentNames = br.readLine().split(separators);
		int n = treatmentNames.length;
		treatments = new MyTallyInArray[n];
		for(int i=0;i<n;i++)	(treatments[i]=new MyTallyInArray()).setName(treatmentNames[i]);
		String line;
		while((line=br.readLine())!=null)
		{
			String[] numberImages = line.split(separators); 
			for(int i=0;i<numberImages.length;i++)
			{
				String image = numberImages[i];
				if(image.equals("")) continue;
				treatments[i].add( Double.parseDouble(image));
			}
		}		
	}
	
	private int unsetTreatments;
	public StatAnalysis(int nTreatments)
	{	
		unsetTreatments = nTreatments;
		treatments = new MyTallyInArray[nTreatments];
	}

	private void addTreatment(String name, BufferedReader br)
	{
		unsetTreatments--;
		MyTallyInArray tally = treatments[unsetTreatments]= new MyTallyInArray();
		tally.setName(name);
		String line;
		try{
			while((line=br.readLine())!=null&& !line.equals(""))
				tally.add(Double.parseDouble(line));
		}catch(IOException ex){ex.printStackTrace();}		
	}
	
	public void addTreatment(MyTallyInArray sample)
	{
		unsetTreatments--;
		treatments[unsetTreatments]=sample;
	}	
}
