package sim.util.stats;
import ec.util.*;
import sim.util.*;
import umontreal.iro.lecuyer.probdist.*;
/**
 * 
 * It offers quantile capabilities. So if you have the data already sorted, feed it in in ASCENDING order!!!
 * 
 * <p>In the "confidence interval of the median", the ranks are
 * already sorted, but I got to have something more general
 * without compromizing the good time-complexity of the 
 * best-case (yet the only one I need).
 * 
 * <p>
 * <font color="red">
 * This should extend TallyInArray, but I can't use it without 
 * adding a "cern" package, </font> so I'll replace that functionality
 * with our own Tally-in-a-Bag, a name with a very nice ring to it.
 *
 * @author Gabriel Balan
 */
public class TallyInABag extends umontreal.iro.lecuyer.stat.Tally
{
	DoubleBag data;
	boolean sorted = true;//ASCENDING
	
	public TallyInABag(){super();data = new DoubleBag();}
	public TallyInABag(String name){super(name);data = new DoubleBag();}
	public TallyInABag(int initialCapacity){super(); data = new DoubleBag(initialCapacity);}
	
	public TallyInABag(DoubleBag db){super(); data = db; db.clear();}
	//cause TallyInArray(DoubleArrayLista) wants a to be empty. Beats me why.	
	
	public void init()
	{
		super.init();
		if(data!=null)//it seems that super() calls init() before data has a chance to be initialized
			data.clear();
		sorted=true;
	}

	public void add(double d)
	{
		if(sorted && data.numObjs>0)
		{
			double last = data.objs[data.numObjs-1];
			sorted = (last<=d);
		}//I don't want to do insertion sort cause that has a quadratic worst case complexity. 
		data.add(d);
		super.add(d);			
	}
	
	public DoubleBag getDataBag(){return data;}
	
	// If u is not a multiple of 1/(n - 1), PERCENTILE interpolates to determine the value at the kth percentile.
	public double quantile(double u)
	{
		if(!sorted) sort();
		if(u<0 || u>1) throw new RuntimeException("Invalid quantile "+u);
		u*=(data.numObjs-1);
		int index1 = (int)u;
		double w1 = u - index1;//frac{u}
		if(w1 ==0) return data.objs[index1];
		int index2 = index1+1;
		double w2 = (w1==0)? 0 : index2 - u;
		return data.objs[index1]*w2 + w1 * data.objs[index2];
	}

	public Object clone()
	{
		TallyInABag tallyinabag = (TallyInABag)super.clone();
		try{tallyinabag.data = (DoubleBag)data.clone();}catch(CloneNotSupportedException ex){ex.printStackTrace();}
		return tallyinabag;
	}

	
	//I copied these from Sean Luke's ec.util.QuickSort, cause it's private over there.
	static private int MAXSTACKSIZE = 1000;	
	static private int THRESHOLD = 10;//off the top of my head.	
	public void sort()
	{
		if(sorted) return;
		sorted=true;
		int oi=0; 
		int oj=data.numObjs-1;
		double[] array = data.objs;
		int[] stack = new int[MAXSTACKSIZE]; // Stack for array bounds
		int listsize = oj-oi+1;
		int top = -1;
		double pivot;
		int pivotindex, l, r;
		double tmp;
	
		stack[++top] = oi;  // Initialize stack
		stack[++top] = oj;

		while (top > 0)    // While there are unprocessed subarrays
		{
			// Pop stack
			int j = stack[top--];
			int i = stack[top--];
	      
			// Findpivot //not radomized, shame on me.
			pivotindex = (i+j)/2;
			pivot = array[pivotindex];
			// Stick pivot at end
			tmp = array[pivotindex]; array[pivotindex] = array[j]; array[j] = tmp; 
			// Partition
			l = i-1;
			r = j;
			do 
			{
				while (array[++l] < pivot);
				while ((r!=0) && (array[--r] > pivot));
				//DSutil.swap(array, l, r);
				tmp = array[l]; array[l] = array[r]; array[r] = tmp;
			} while (l < r);
			// Undo final swap
			tmp = array[l]; array[l] = array[r]; array[r] = tmp;
			// Put pivot value in place
			tmp = array[l]; array[l] = array[j]; array[j] = tmp;
		    
			// Put new subarrays onto stack if they are small
			if ((l-i) > THRESHOLD)   // Left partition
			{ 
				stack[++top] = i;
				stack[++top] = l-1;
			}
			if ((j-l) > THRESHOLD) // Right partition 
			{   
				stack[++top] = l+1;
				stack[++top] = j;
			}
		}
		//final insertion sort.
		for (int i=1; i<data.numObjs; i++) // Insert i'th record
			for (int j=i; (j>0) && (array[j]<array[j-1]); j--)
			//DSutil.swap(array, j, j-1);
			{ tmp = array[j]; array[j] = array[j-1]; array[j-1] = tmp; }
	}
	
	/**
	 * Thomson-Savur: confidence interval around the median.
	 * 
	 * According to Wineberg & Christensen (GECCO 03).
	 * binomial quantile /w cumulatice upper
	 * tail = alpha/2		b = excel's CRITBINOM(n, 0.5, alpha/2)
	 * lower percentile		l = b/(n-1) 
	 * upper percentile 	u = 1-l
	 * lower bound			value_l = excel's PERCENTILE(data, l)
	 * upper bound			value_u = excel's PERCENTILE(data, u)
	 * 
	 * Advantages:		It gives a better understandind of the system
	 * 						-median  /w error bounds
	 * 						-easy to draw on a graph.
	 * (my own:				-I can compare more than 2 treatments without n^2/2 re-rankings, like the ranked T-test)
	 * 
	 * Disadvantage:	Not as sensitive as a ranked T test
	 */
	public void  confidenceIntervalMedian(double alpha, double[] centerAndRadius)
	{
		int n  = data.numObjs;
		double b = BinomialDist.inverseF(n, .5, alpha/2);

//		double l = b/(n-1);
//		double u = 1-l;
//		double value_l = quantile(l);
//		double value_u = quantile(u);
//		double center = centerAndRadius[0]=(value_l+value_u)/2;
//		centerAndRadius[1] = center - value_l;
		
		//the quantile of a multiple of 1/(n-1) is one of the sample values (no interpolation)
		//so here is the same thing, but faster.
		if(!sorted) sort();
		int b_fast = (int)b;
		double value_l_fast = data.objs[b_fast];
		double value_u_fast = data.objs[n-1-b_fast];
		double center = centerAndRadius[0]=(value_l_fast+value_u_fast)/2;
		centerAndRadius[1] = center - value_l_fast;
	}
	

	/**
	 * Tally.confidenceIntervalStudent uses the confidence coefficient!
	 * TallyInABag.confidenceIntervalMean uses the significance level
	 * 
	 * This function borrows the meaning from EXCEL:
	 * 			"Alpha is the significance level used to compute the confidence level. 
	 * 			The confidence level equals 100*(1 - alpha)%, or in other words, 
	 * 			an alpha of 0.05 indicates a 95 percent confidence level."

	 * @param alpha = significance level
	 * 1-alpha = confidence coefficient
	 * 100(1-alpha) = confidence level
	 * @param centerAndRadius
	 */
	public void  confidenceIntervalMean(double alpha, double[] centerAndRadius)
	{
		centerAndRadius[0]=average();

		int n  = data.numObjs;
		double d1 = StudentDist.inverseF(n - 1, 1- 0.5* alpha);
		centerAndRadius[1] = d1 * Math.sqrt(variance() / (double)n);
	}
	
	public void print(){print(0.95);}

	public void print(double confidenceLevel)
	{
		System.out.println("\nStats for "+getName());
		System.out.println("SampleSize...\t"+data.numObjs);
		System.out.println("Average......\t"+average());
		if(data.numObjs>=2)
			System.out.println("Stdev........\t"+standardDeviation());
		System.out.println("Min..........\t"+minValue);
		System.out.println("Max..........\t"+maxValue);		
		double[] centerAndRadius = new double[2];
		if(data.numObjs>=2)
		{
			confidenceIntervalMean(confidenceLevel, centerAndRadius);
			System.out.println(confidenceLevel+"-Confidence interval for the mean:\t" + (centerAndRadius[0]-centerAndRadius[1]) + "\t" + (centerAndRadius[0]+centerAndRadius[1]));
			confidenceIntervalMedian(confidenceLevel, centerAndRadius);
			System.out.println(confidenceLevel+"-Confidence interval for the median:\t" + (centerAndRadius[0]-centerAndRadius[1]) + "\t" + (centerAndRadius[0]+centerAndRadius[1]));	
		}
	}

	public static TallyInABag pairwiseMax(TallyInABag t1, TallyInABag t2)
	{
		int n = t1.numberObs(); 
		assert(n ==t2.numberObs());
		TallyInABag t = new TallyInABag(n);
		for(int i=0;i<n;i++)
			t.add(Math.max(t1.data.objs[i], t2.data.objs[i]));
		return t;
	}

	public static final void main(String[] args)
	{
//		//instead of CRITBINOM
//		//the static call is doing linear search
//		//the other one does binary search.
//		//although the treatments have usually the same size, 
//		//unfortunatelly the code is more general. On the other hand, 
//		//speed is not a issue, even in the online case (racing).
//		BinomialDist bd = new BinomialDist(10, 1d/2);
//		double p = 0;
//		for(int i=0;i<10;i++)
//		{
//			System.out.println(p+"\t"+bd.inverseF(p)+"\t"+ BinomialDist.inverseF(10, .5, p));
//			p+=0.1;
//		}
		TallyInABag tb = new TallyInABag();
		MersenneTwisterFast rng = new MersenneTwisterFast();
		for(int i=0;i<100;i++)
			tb.add(rng.nextDouble());
		tb.confidenceIntervalMedian(0.05, new double[2]);
	}
}
