package sim.util.stats.doubletime;
import sim.engine.*;
/**
 * @author Catalin Balan
 *
 * Computes the average of a piece-wise constant function 
 * defined double->int. E.g. the average queue length.
 * 
 * O(1) memory requirements.
 * If you want stdev too, this is the place to add the code.
 */
public class TimeWeightedAverager
{
	//	the very start
	public double start;
	
	//	the start of the last level
	double levelStartingTime;
	
	public double sum;
	Schedule timer;	
	int level;
	int max;
	
	public TimeWeightedAverager(Schedule schedule){this(schedule, 0);}
	public TimeWeightedAverager(Schedule schedule, int firstLevel)
	{
		timer= schedule;
		max = level = firstLevel;
	}

	//this resets the statistics, but the level remains unchanged.
	public void start()
	{
		levelStartingTime = start = timer.time();
		sum = 0;
		max = level;
		bookmark=start;
		preBookmarkSum=0;
	}
	//sample is the level for the interval that starts now.
	public void addSample(int sample)
	{
		double now = timer.time();
		if(now==start)
			max = sample;
		else
		{
			sum+=level*(now-levelStartingTime);//saved the info on the last interval
			if(level>max)
				max = level;
		}
		level = sample;//prepare the next one
		levelStartingTime = now;
	}
	
	
	
	/**
	 * Dirty little loophole. One should at least try not to use it.
	 */
	public void performCorrection(double diff){sum+=diff;}
	
	public double getAverage()
	{
		addSample(level);
		double now = timer.time();
		return sum/(now-start);
	}
	
	public int getMax()
	{
		addSample(level);
		return max;
	}
	
	double bookmark=0;
	double preBookmarkSum=0;
	public void setBookMark()
	{
		preBookmarkSum = sum;
		bookmark = timer.time();
	}
	public double getPostBookMarkAverage()
	{
		addSample(level);
		double now = timer.time();
		return (sum-preBookmarkSum)/(now-bookmark);
	}
}
