package sim.util.stats.doubletime;
import sim.engine.*;
/**
 * @author Catalin Balan
 *
 * Computes the average of a piece-wise constant function 
 * defined double->double.
 * 
 * O(1) memory requirements.
 * If you want stdev too, this is the place to add the code.
 */
public class TimeWeightedDAverager
{
	//	the very start
	double start;
	
	//	the start of the last level
	double levelStartingTime;
	
	double sum;
	Schedule timer;	
	double level;
	double max;
	
	public TimeWeightedDAverager(Schedule schedule){this(schedule, 0d);}
	public TimeWeightedDAverager(Schedule schedule, double firstLevel)
	{
		timer= schedule;
		max = level = firstLevel;
	}

	//this resets the statistics, but the level remains unchanged.
	public void start()
	{
		levelStartingTime = start = timer.time();
		sum = 0;
		max = level;
	}
	//sample is the level for the interval that starts now.
	public void addSample(double sample)
	{
		double now = timer.time();
		if(now==start)
			max = sample;
		else
		{
			sum+=level*(now-levelStartingTime);//saved the info on the last interval
			if(level>max)
				max = level;
		}
		level = sample;//prepare the next one
		levelStartingTime = now;
	}
	
	public double getAverage()
	{
		addSample(level);
		double now = timer.time();
		return sum/(now-start);
	}
	
	public double getMax()
	{
		addSample(level);
		return max;
	}
}
